"""
A general-purpose Simulation class for discrete events.  

The Simulation object is the central object of a simulation.  It
handles the simulation clock time and maintains communication between
the components of the simulation.

A simulation is structured as a directed graph of event-processing
nodes called EventProcessors (EPs).  EventProcessors generate
data-carrying Events, which are routed through the graph to other
EventProcessors via delayed connections.  Most objects in the
simulation will be a subclass of EventProcessor, customized to provide
some specific behavior.

There are different subclasses of Event, each doing different types of
computation, and each can contain any arbitrary Python data.  A
simulation begins by giving each EventProcessor an opportunity to send
any initial events.  It then proceeds by processing and delivering
events to EventProcessors in time order.  After all events for the
current time are processed, the simulation gives each EventProcessor a
chance to do any final computation, after which simulation time skips
to the time of the earliest event remaining in the queue.


PORTS

All connections between EPs are tagged with a source port and a
destination port.  Ports are internal addresses that EPs can use to
distinguish between inputs and outputs.  A port specifier can be any
hashable Python object.  If not specified, the input and output ports
for a connection default to None.

src_ports distinguish different types of output that an EP may
produce. When sending output, the EP must call self.send_output()
separately for each port.  dest_ports distinguish different types of
input an EP may receive and process.  The EP is free to interpret the
input port designation on an incoming event in any way it chooses.

An example dest_port might be for an EP that receives 'ordinary'
neural activity on the default port, and receives a separate
modulatory signal that influences learning.  The originator of the
modulatory signal might have a connection to the EP with dest_port =
'Modulation'.  Multiple ports can be grouped by a dest EP by assuming
a convention that the keys are tuples,
e.g. ('JointNormalize','Group1'), ('JointNormalize','Group2').

An example src_port use might be an EP that sends different events to
itself than it sends out to other EPs.  In this case the self
connections might have src_port = 'Recurrent', and probably also a
special dest_port.


$Id$
"""

import param

from copy import copy, deepcopy
import time
import bisect

# CEBALERT: Is it dangerous to have 'Forever' implemented
# like this? To start with, min(Forever,1) gives
# FixedPoint('-1.00,2') i.e. Forever.
# Forever = FixedPoint(-1)
Forever = -1
# (Python 2.6 includes support for float('inf') on all platforms?)

#: Default path to the current simulation, from main
#: Only to be used by script_repr(), to allow it to generate
#: a runnable script
_simulation_path="topo.sim"




class EventProcessor(param.Parameterized):
    """
    Base class for EventProcessors, i.e. objects that can accept and
    handle events.  This base class handles the basic mechanics of
    connections and sending events, and stores both incoming and outgoing
    connections.

    The dest_ports attribute specifies which dest_ports are supported
    by this class; subclasses can augment or change this list if they
    wish.  The special value dest_ports=None means to accept
    connections to any dest_port, while dest_ports=[None,'Trigger']
    means that only connections to port None or port 'Trigger' are
    accepted.

    Similarly, the src_ports attribute specifies which src_ports will
    be given output by this class.
    """
    __abstract = True

    dest_ports=[None]

    src_ports=[None]
    

    def __init__(self,**params):
        """
        Create an EventProcessor.

        Note that just creating an EventProcessor does not mean it is
        part of the simulation (i.e. it is not in the simulation's list
        of EventProcessors, and it will not have its start() method called).
        To add an EventProcessor e to a simulation s, simply do
        s['name_of_e']=e. At this point, e's 'name' attribute will be set
        to 'name_of_e'.
        """
        super(EventProcessor,self).__init__(**params)

        # A subclass could use another data stucture to optimize operations
        # specific to itself, if it also overrides _dest_connect().
        self.in_connections = []
        self.out_connections = []

        self.simulation = None

    def _port_match(self,key,portlist):
        """
        Returns True if the given key matches any port on the given list.

        In the default implementation, a port is considered a match if
        the port is == to the key, but subclasses of EventProcessor can
        override this to allow weaker forms of matching.
        """
        return key in portlist

    # if extra parameters are required for an EP subclass, a
    # dictionary could be added to Simulation.connect() to hold
    # them, and passed on here
    def _src_connect(self,conn):
        """
        Add the specified connection to the list of outgoing connections.
        Should only be called from Simulation.connect().
        """

        if self.src_ports and not self._port_match(conn.src_port,self.src_ports):
            raise ValueError("%s is not on the list of ports provided for outgoing connections for %s: %s." %
                             (str(conn.src_port), self.__class__, str(self.src_ports)))

        # CB: outgoing connection must be uniquely named among others
        # going to the same destination.
        for existing_connection in self.out_connections:
            if existing_connection.name==conn.name and existing_connection.dest==conn.dest:
                raise ValueError('A connection out of an EventProcessor must have a unique name among connections to a particular destination; "%s" out of %s into %s already exists'%(conn.name,conn.dest,self.name))

        # CB: alternative: outgoing connection must have a unique name
##         for existing_connection in self.out_connections:
##             if existing_connection.name==conn.name:
##                 raise ValueError('A connection out of an EventProcessor must have a unique name; "%s" out of %s already exists'%(conn.name,self.name))
  
        self.out_connections.append(conn)


    def _dest_connect(self,conn):
        """
        Add the specified connection to the list of incoming connections.
        Should only be called from Simulation.connect().
        """

        if self.dest_ports and not self._port_match(conn.dest_port,self.dest_ports):
            raise ValueError("%s is not on the list of ports allowed for incoming connections for %s: %s." %
                             (str(conn.dest_port), self.__class__, str(self.dest_ports)))

        for existing_connection in self.in_connections:
            if existing_connection.name == conn.name:
                raise ValueError('A connection into an EventProcessor must have a unique name; "%s" into %s already exists'%(conn.name,self.name))

        self.in_connections.append(conn)


    def __dir__(self):
        """
        Extend dir() to include in_connections of an EventProcessor.
        Useful for software that examines the list of possible
        in_connections, such as tab completion in IPython.
        """
        default_dir = dir(type(self)) + list(self.__dict__)
        return sorted(set(default_dir + [conn.name for conn in self.in_connections]))


    def __getattr__(self, name):
        """
        Provide a simpler attribute-like syntax for accessing the
        in_connections of an EventProcessor (e.g. obj.conn, for
        in_connection "conn" of EventProcessor obj).
        """
        for conn in self.in_connections:
            if conn.name == name: return conn
        raise AttributeError


    def start(self):
        """        
        Called by the simulation when the EventProcessor is added to
        the simulation.

        If an EventProcessor needs to have any code run when it is
        added to the simulation, the code can be put into this method
        in the subclass.
        """
        pass

    ### JABALERT: Should change send_output to accept a list of src_ports, not a single src_port.
    def send_output(self,src_port=None,data=None):
        """
        Send some data out to all connections on the given src_port.
        The data is deepcopied before it is sent out, to ensure that
        future changes to the data are not reflected in events from
        the past.
        """
        
        out_conns_on_src_port = [conn for conn in self.out_connections
                                 if self._port_match(conn.src_port,[src_port])]

        data=deepcopy(data)
        for conn in out_conns_on_src_port:
            #self.verbose("Sending output on src_port %s via connection %s to %s" % (str(src_port), conn.name, conn.dest.name))
            e=EPConnectionEvent(self.simulation.convert_to_time_type(conn.delay)+self.simulation.time(),conn,data,deep_copy=False)
            self.simulation.enqueue_event(e)
            

    def input_event(self,conn,data):
        """
        Called by the simulation when an EPConnectionEvent is delivered;
        the EventProcessor should process the data somehow.
        """
        raise NotImplementedError


    def process_current_time(self):
        """
        Called by the simulation before advancing the simulation
        time.  Allows the event processor to do any computation that
        requires that all events for this time have been delivered.
        Computations performed in this method should not generate any
        events with a zero time delay, or else causality could be
        violated. (By default, does nothing.)
        """
        pass
    
    def script_repr(self,imports=[],prefix="    "):
        """Generate a runnable command for creating this EventProcessor."""
        return _simulation_path+"['"+self.name+"']="+\
        super(EventProcessor,self).script_repr(imports=imports,prefix=prefix)



        
class EventProcessorParameter(param.Parameter):
    """Parameter whose value can be any EventProcessor instance."""
     
    def __init__(self,default=EventProcessor(),**params):
        super(EventProcessorParameter,self).__init__(default=default,**params)
        
    def __set__(self,obj,val):
        if not isinstance(val,EventProcessor):
            raise ValueError("Parameter must be an EventProcessor.")
        else:
            super(EventProcessorParameter,self).__set__(obj,val)


from param import parameterized

class EPConnection(param.Parameterized):
    """
    EPConnection stores basic information for a connection between
    two EventProcessors.
    """

## JPALERT: This type-checking is redundant, since
##     Simulation.connect() only allows the user to create connections
##     between existing simulation objects, which must be EPs.  Type
##     checking here means that it is impossible to ever instantiate
##     an EPConnection in any situation (including debugging) w/o
##     making src and dest be EPs.  However, there is nothing I can
##     find that requires that the src or dest be EPs.  While some
##     *subclasses* of EPConnection (such as Projection) do require
##     that their src and dest support the interfaces of some
##     *subclasses* of EventProcessor (e.g. Sheet.activity), there is
##     no reason that those objects have to be EPs, per se.  IMO,
##     excessive type checking removes much of the power of using a
##     dynamic language like Python.
    
##     src = EventProcessorParameter(default=None,constant=True,precedence=0.10,doc=
##        """The EventProcessor from which messages originate.""")
    
##     dest = EventProcessorParameter(default=None,constant=True,precedence=0.11,doc=
##        """The EventProcessor to which messages are delivered.""")

    src = param.Parameter(default=None,constant=True,precedence=0.10,doc=
       """The EventProcessor from which messages originate.""")
    
    dest = param.Parameter(default=None,constant=True,precedence=0.11,doc=
       """The EventProcessor to which messages are delivered.""")
    
    src_port = param.Parameter(default=None,precedence=0.20,doc=
       """
       Identifier that can be used to distinguish different types of outgoing connections.

       EventProcessors that generate only a single type of
       outgoing event will typically use a src_port of None.  However,
       if multiple types of communication are meaningful, the
       EventProcessor can accept other values for src_port.  It is up
       to the src EventProcessor to deliver appropriate data to each
       port, and to declare what will be sent over that port.
       """)
    
    dest_port = param.Parameter(default=None,precedence=0.21,doc=
       """
       Identifier that can be used to distinguish different types of incoming connections.

       EventProcessors that accept only a single type of incoming
       event will typically use a src_port of None.  However, if
       multiple types of communication are meaningful, the
       EventProcessor can accept other values for dest_port.  It is up
       to the dest EventProcessor to process the data appropriately
       for each port, and to define what is expected to be sent to
       that port.
       """)

    # Should the lower bound be exclusive?
    delay = param.Number(default=0.05,bounds=(0,None),doc="""
       Simulation time between generation of an Event by the src and delivery to the dest.
       Should normally be nonzero, to represent a causal with a well-defined ordering
       of events.""")

    private = param.Boolean(default=False,doc=
       """Set to true if this connection is for internal use only, not to be manipulated by a user.""")


    # CEBALERT: should be reimplemented. It's difficult to understand,
    # and contains the same code twice. But it does work.
    def remove(self):
        """
        Remove this connection from its src's list of out_connections and its
        dest's list of in_connections.
        """
        # remove from EPs that have this as in_connection
        i = 0
        to_del = []
        for in_conn in self.dest.in_connections:
            if in_conn is self:
                to_del.append(i)
            i+=1

        for i in to_del:
            del self.dest.in_connections[i]

        # remove from EPs that have this as out_connection
        i = 0
        to_del = []
        for out_conn in self.src.out_connections:
            if out_conn is self:
                to_del.append(i)
            i+=1

        for i in to_del:
            del self.src.out_connections[i]


    def script_repr(self,imports=[],prefix="    "):
        """Generate a runnable command for creating this connection."""

        if self.private:
            return ""
        
        settings=[]
        for name,val in self.get_param_values():
            try: # There may be a better way to figure out which parameters specify classes
                if issubclass(val,object):
                    rep=val.__name__
                    # Generate import statement
                    cls = val.__name__
                    mod = val.__module__
                    imports.append("from %s import %s" % (mod,cls))
            except TypeError:
                if name=="src" or name=="dest":
                    rep=None
                else:
                    rep = parameterized.script_repr(val,imports,prefix,settings)

            if rep is not None:
                settings.append('%s=%s' % (name,rep))

        # add import statement
        cls = self.__class__.__name__
        mod = self.__module__
        imports.append("from %s import %s" % (mod,cls))

        return _simulation_path+".connect('"+self.src.name+"','"+self.dest.name+ \
               "',connection_type="+self.__class__.__name__+ \
               ",\n"+prefix+(",\n"+prefix).join(settings) + ")"


# CB: event is not a Parameterized because of a (small) performance hit.
class Event(object):
    """Hierarchy of classes for storing simulation events of various types."""

    def __init__(self,time):
        self.time = time

    def __call__(self,sim):
        """
        Cause some computation to be performed, deliver a message, etc.,
        as appropriate for each subtype of Event.  Should be passed the
        simulation object, to allow access to .time() etc.        
        """
        raise NotImplementedError

    def __cmp__(self,ev):
        """
        Implements event comparison by time, allowing sorting,
        and queue maintenance  using bisect module or minheap
        implementations, if needed.

        NOTE: identity comparisons should always be done using the
        'is' operator, not '=='.
        """
        if self.time > ev.time:
            return 1
        elif self.time < ev.time:
            return -1
        else:
            return 0


class EPConnectionEvent(Event):
    """
    An Event for delivery to an EPConnection.

    Provides access to a data field, which can be used for anything
    the src wants to provide, and a link to the connection over which
    it has arrived, so that the dest can determine what to do with the
    data.

    By default, the data is deepcopied before being added to this
    instance for safety (e.g. so that future changes to data
    structures do not affect messages arriving from the past).
    However, if you can ensure that the copying is not
    necessary (e.g. if you deepcoy before sending a set of
    identical messages), then you can pass deep_copy=False
    to avoid the copy.
    """
    
    def __init__(self,time,conn,data=None,deep_copy=True):
        super(EPConnectionEvent,self).__init__(time)
        assert isinstance(conn,EPConnection)
        self.data = deepcopy(data) if deep_copy else data
        self.conn = conn

    def __call__(self,sim):
        self.conn.dest.input_event(self.conn,self.data)

    def __repr__(self):
        return "EPConnectionEvent(time="+`self.time`+",conn="+`self.conn`+")"


class CommandEvent(Event):
    """An Event consisting of a command string to execute."""

    def __init__(self,time,command_string):
        """
        Add the event to the simulation.

        Raises an exception if the command_string contains a syntax
        error.
        """
        self.command_string = command_string
        self.__test()
        super(CommandEvent,self).__init__(time)
        
    def __repr__(self):
        return "CommandEvent(time="+`self.time`+", command_string='"+self.command_string+"')"


    def script_repr(self,imports=[],prefix="    "):
        """Generate a runnable command for creating this CommandEvent."""
        return _simulation_path+'.schedule_command('\
               +`self.time`+',"'+self.command_string+'")'


    def __call__(self,sim):
        """
        exec's the command_string in __main__.__dict__.
        
        Be sure that any required items will be present in
        __main__.__dict__; in particular, consider what will be present
        after the network is saved and restored. For instance, results of
        scripts you have run, or imports they make---all currently
        available in __main__.__dict__---will not be saved with the
        network.
        """
        # Presumably here to avoid importing __main__ into the rest of the file
        import __main__

        param.Parameterized(name='CommandEvent').message("Running command %s" \
                                                         % (self.command_string))
        try:
            exec self.command_string in __main__.__dict__
        except:
            print "Error in scheduled command:"
            raise
            
    def __test(self):
        """
        Check for SyntaxErrors in the command.
        """
        try:
            compile(self.command_string,"CommandString","single") 
        except SyntaxError:
            print "Error in scheduled command:"
            raise
        

class FunctionEvent(Event):
    """
    Event that executes a given function function(*args,**kw).
    """
    def __init__(self,time,fn,*args,**kw):
        super(FunctionEvent,self).__init__(time)
        self.fn = fn
        self.args = args
        self.kw = kw

    def __call__(self,sim):
        self.fn(*self.args,**self.kw)

    def __repr__(self):
        return 'FunctionEvent(%s,%s,*%s,**%s)' % (`self.time`,`self.fn`,`self.args`,`self.kw`)

class EventSequence(Event):
    """
    Event that contains a sequence of other events to be scheduled and
    executed.

    The .time attributes of the events in the sequence are interpreted
    as offsets relative to the start time of the sequence itself.
    """
    def __init__(self,time,sequence):
        super(EventSequence,self).__init__(time)
        self.sequence = sequence

    def __call__(self,sim):

        # Enqueue all the events in the sequence, offsetting their
        # times from the current time
        sched_time = sim.time()
        for ev in self.sequence:
            new_ev = copy(ev)
            sched_time += ev.time
            new_ev.time = sched_time
            sim.enqueue_event(new_ev)

    def __repr__(self):
        return 'EventSequence(%s,%s)' % (`self.time`,`self.sequence`)



class PeriodicEventSequence(EventSequence):
    """
    An EventSequence that reschedules itself periodically

    Takes a period argument that determines how often the sequence
    will be scheduled.   If the length of the sequence is longer than
    the period, then the length of the sequence will be used as the period.
    """
    ## JPHACKALERT: This should really be refactored into a
    ## PeriodicEvent class that periodically executes a single event,
    ## then the user can construct a periodic sequence using a
    ## combination of PeriodicEvent and EventSequence.  This would
    ## change the behavior if the sequence length is longer than the
    ## period, but I'm not sure how important that is, and it might
    ## actually be useful the other way.
    

    def __init__(self,time,period,sequence):
        super(PeriodicEventSequence,self).__init__(time,sequence)
        self.period = period
        
    def __call__(self,sim):        
        super(PeriodicEventSequence,self).__call__(sim)

        # Find the timed length of the sequence
        seq_length = sum(e.time for e in self.sequence)

        if seq_length < self.period:
            # If the sequence is shorter than the period, then reschedule
            # the sequence to occur again after the period
            self.time += self.period
        else:
            # If the sequence is longer than the period, then
            # reschedule to start after the sequence ends.
            self.time += seq_length
        sim.enqueue_event(self)

    def __repr__(self):
        return 'PeriodicEventSequence(%s,%s,%s)' % (`self.time`,`self.period`,`self.sequence`)


# CB: code that previously existed in various places now collected
# together. The original timing code was not properly tested, and the
# current code has not been tested either: needs writing cleanly and
# testing. This whole class is pretty difficult to follow.
#  
### JP: Is it possible that some or all of this can be more cleanly
### implemented using PeriodicEvents?

from math import floor
class SomeTimer(param.Parameterized):
    """
    Provides a countdown timer for functions that run repeatedly.
    
    There are two distinct ways to use the timer.

    The first, via call_and_time(), is for calling some function every
    specified number of steps for a specified duration. Currently
    call_and_time() is used for timing calls to simulation.run() every
    1.0 steps for 100 iterations. See the Simulation class for an
    example of using the timer in this way.

    The second, via call_fixed_num_times(), is for calling some
    function repeatedly a specified number of times. A case to use
    call_fixed_num_times() would be timing pattern presentations,
    where the number of times the pattern_presenter will be called is
    known in advance. Additionally, this method allows a list of
    arguments to be passed to the function (in this case, the
    permutation for each call).
    """
    # * parameters to control formatting?
    # * the parameter types for some of the following could be more specific
    step = param.Parameter(default=2,doc=
        """Only relevant with call_and_time(), not call_fixed_num_times(). 
        
           Each iteration, func is called as func(step).
    
           For example, step=1 with func set to topo.sim.time would cause
           the simulation time to advance once per iteration.

           The default value (None) gives 50 iterations for any value of simulation_duration
           passed to call_and_time(simulation_duration).""")
    
    estimate_interval = param.Number(default=50,doc=
        """Interval in simulation time between estimates.""") 

    func = param.Parameter(default=None,instantiate=True,doc=
        """Function to be timed.""")

##     func_args = Parameter(default=None,instantiate=True,doc=
##         """Arguments passed to func at time of calling.""")

    simulation_time_fn = param.Parameter(default=None,instantiate=True,doc=
        """Function that returns the simulation time.""")

    real_time_fn = param.Parameter(default=time.time,instantiate=True,doc=
        """Function that returns the wallclock time.""")

    receive_info = param.Parameter(default=[],instantiate=True,doc=
        """List of objects that will receive timing information.
        Each must have a timing_info() method.""")
                                 
    stop = param.Boolean(default=False,doc=
        """If set to True, execution of func (and timing) will cease at the end of
        the current iteration.""")


    def __pass_out_info(self,time,percent,name,duration,remaining):
        [thing(time,percent,name,duration,remaining) for thing in self.receive_info]

    # CB: this used to say how long the operation took (in wallclock time)

    def __measure(self,fduration,step,arg_list=None):

        if not arg_list:
            # no list of arguments means not being called set number of times
            fixed_num_calls = False
        else:
            fixed_num_calls = True
            
        iters  = int(floor(fduration/step))

        recenttimes=[]

        if not fixed_num_calls: arg_list=[step]*iters
        
        simulation_starttime = self.simulation_time_fn()

        self.stop = False
        for i in xrange(iters):
            recenttimes.append(self.real_time_fn())
            length = len(recenttimes)

            if (length>self.estimate_interval):
                recenttimes.pop(0)
                length-=1

            self.func(arg_list[i])

            percent = 100.0*i/iters
            estimate = (iters-i)*(recenttimes[-1]-recenttimes[0])/length
            self.__pass_out_info(time=self.simulation_time_fn(),
                                 percent=percent,
                                 name=self.func.__name__,
                                 duration=fduration,
                                 remaining=estimate)
            
            ## HACK refresh windows for camera in simulation time
            import topo 
            if hasattr(topo, 'guimain'):
                topo.guimain.refresh_activity_windows()
            ##

            if self.stop: break


        if not self.stop:
            if not fixed_num_calls:
                # ensure specified duration has been respected, since code above might not
                # complete specified duration (integer number of iterations)
                leftover = fduration+simulation_starttime-self.simulation_time_fn()
                if leftover>0: self.func(leftover)
            percent = 100   
        self.__pass_out_info(time=self.simulation_time_fn(),
                             percent=percent,
                             name=self.func.__name__,
                             duration=fduration,
                             remaining=0)


              
    def call_fixed_num_times(self,args_for_iterations):
        """
        Call self.func(args_for_iterations[i]) for all i in args_for_iterations.
        """
        self.__measure(len(args_for_iterations),1.0,arg_list=args_for_iterations)
        
    
    def call_and_time(self,fduration):
        """
        Call self.func(self.step or fduration/50.0) for fduration.
        """
        # default to 50 steps unless someone set otherwise
        step = self.step or fduration/50.0
        self.__measure(fduration,step)



# CEBALERT: This singleton-producing mechanism is pretty complicated,
# and it would be great if someone could simplify it. Getting all of
# the behavior we want for e.g. Simulation is tricky, but there are
# tests for it. Note that:
# (1) There should only ever be one single Simulation instance for
#     which register is True. Creating, copying, and unpickling
#     need to take this into account.
# (2) A Simulation instance for which register is False should
#     behave the same as any normal Python object.
#
# For how to use, see topo.base.simulation.Simulation or
# topo.misc.commandline.GlobalParams.
class OptionalSingleton(object):

    _inst = None

    def __new__(cls,singleton):
        """
        Return the single instance stored in _inst if singleton is
        True; otherwise, return a new instance.
        """
        if singleton:
            if cls is not type(cls._inst):
                cls._inst = object.__new__(cls) 
                cls._inst._singleton = True
            return cls._inst
        else:
            new_inst = object.__new__(cls) 
            new_inst._singleton = False
            return new_inst

    def __getnewargs__(self):
        return (self._singleton,)

    def __copy__(self):
        # An OptionalSingleton(singleton=False) instance is copied, while the
        # OptionalSingleton(singleton=True) instance is not copied.
        if self._singleton:
            return self
        else:
            # Ideally we'd just call "object.__copy__", but apparently
            # there's no such method.
            
            # CB: I *think* this is how to do a copy. Any better
            # ideas?  Python's copy.copy() function calls an object's
            # __reduce__ method and then reconstructs the object from
            # that using copy._reconstruct().
            new_obj = self.__class__(self._singleton)
            new_obj.__dict__ = copy(self.__dict__)
            return new_obj 

    def __deepcopy__(self,m):
        if self._singleton:
            return self
        else:
            new_obj = self.__class__(self._singleton)
            new_obj.__dict__ = deepcopy(self.__dict__,m)
            return new_obj

    # CB: I might have bound __copy__ (& __deepcopy__) just to the
    # Simulation(singleton=True) instance to avoid the Simulation
    # class having a __copy__ method at all, but copy() only checks
    # the *class* for the existence of __copy__.



# Simulation stores its events in a linear-time priority queue (i.e., a
# sorted list.) For efficiency, e.g. for spiking neuron simulations,
# we'll probably need to replace the linear priority queue with a more
# efficient one.  Jeff has an O(log N) minheap implementation, but
# there are likely to be many others to select from.
#
class Simulation(param.Parameterized,OptionalSingleton):
    """
    A simulation class that uses a simple sorted event list (instead of
    e.g. a sched.scheduler object) to manage events and dispatching.

    Simulation is a singleton: there is only one instance of
    Simulation, no matter how many times it is instantiated.
    """

    # CEBALERT: Must use convert_to_time_type() everywhere a time is
    # specified to ensure all times are the appropriate type. Relying
    # on everyone to convert times to the time type seems dangerous!

    time_type = param.Parameter(default=float,constant=True,doc="""
        Callable for converting user-specified times into the numeric
        type to be used for Simulation's time values.
        
        Simulation's time can be set to any numeric type that supports
        the usual Python numeric operations, including at least
        multiplication, addition, and subtraction.  Most of the code
        assumes that fractional values are supported, as they are for
        floats, but it may be possible to use an integer type instead
        if various default fractional values are overridden.
        
        For instance, one might wish to use arbitrary precision
        floating-point time to avoid accumulating rounding errors.  If
        stepping through a simulation every 0.05 time units, after 20
        such steps using floats the simulation will not reach 1.0
        exactly, but will instead be slightly higher or lower.  With
        an arbitrary precision float type such a series of steps can
        be guaranteed to reach 1.0 exactly.  Alternatively, one might
        wish to use a rational type so that events can be guaranteed
        to happen a certain number of times in a given interval, even
        if the ratio cannot be expressed as an even decimal or binary
        fraction.
                
        time_type determines how the user can specify times. For
        instance, if the user specifies a time as 0.05 (a float), this
        will be represented in floating point as something like
        0.050000000000000003. gmpy.mpq will correctly convert that to
        mpq(1,20). Other types might require times to be specified as
        strings (e.g. decimal.Decimal('0.05') in Python 2.6); in this
        case the user would need to use a string whenever specifying a
        time (e.g. a connection delay or a call to run() ). Some other
        types will only produce the expected result if they are first
        configured appropriately (e.g. decimal.Decimal(0.05) in Python
        2.7 will give something like
        Decimal('0.05000000000000000277555756156289135105907917022705078125')
        by default, but this can be controlled by using an appropriate
        decimal.Context).

        Extra arguments can be passed to time_type when it is being used
        to create the Simulation's time: see time_type_args.


        Some potentially useful number classes::
        
        - gmpy.mpq: gmpy provides Python with access to the fast GNU
          Multi-Precision library (which requires GMP to be built);
          gmpy.mpq is gmpy's rational type.
        
        - fixedpoint.FixedPoint: pure Python fixed-point number, but
          quite slow.
        
        - Python's decimal.Decimal and fractions.Fraction classes.
        """)

    # CEBALERT: probably should remove this and allow people to deal
    # with functions requiring arguments
    # (e.g. fixedpoint.FixedPoint(time,precision=4)) by currying or
    # creating their own wrapper function. We wouldn't then need
    # convert_to_time_type(), it would instead just be a call to
    # time_type() (which should also be renamed).
    time_type_args = param.Parameter(default=(),constant=True,doc="""
        Tuple of arguments passed to time_type after the first
        argument (which is the time).""")

    register = param.Boolean(default=True,constant=True,doc="""
        Whether or not to register this Simulation. If True, this
        Simulation (when created explicitly or when unpickled)
        will replace any existing registered Simulation (if one exists).
        Thus only one Simulation with register=True can exist at
        any one time, which makes it simpler to handle defining
        and reloading a series of simulations without polluting the
        memory space with unused simulations.""")

    startup_commands = param.Parameter(instantiate=True,default=[],doc="""
        List of string commands that will be exec'd in
        __main__.__dict__ (i.e. as if they were entered at the
        command prompt) every time this simulation is unpickled
        (and will be executed before the simulation is itself
        unpickled).

        For example, allows items to be imported before
        scheduled_commands are run.
        """)

    time_printing_format = param.String("%(_time)09.2f",doc="""
        Format string to be used when the simulation time must be
        formatted as a string, e.g. for display or for basename().
        When the string is evaluated, the time will be available as
        the attribute '_time'.
        """)

    basename_format = param.String("%(name)s_%(timestr)s",doc="""
        Format string to be used by the basename() function.
        When the string is evaluated, the formatted time from
        time_printing_format will be available as the attribute
        'timestr'.
        """)
    
    eps_to_start = []

    name = param.Parameter(constant=False)


    ### Simulation(register=True) is a singleton
    #
    #
    # There is only ever one instance of Simulation(register=True).
    # This instance is stored in the '_inst' attribute; when __new__
    # is called and register is True, this instance is created if it
    # doesn't already exist, and returned otherwise. copying or
    # deepcopying this instance returns the instance.  
    #
    # For a Simulation with register False, calling __new__ results in
    # a new object as usual for Python objects. copying and
    # deepcopying returns a new Simulation with a copy or deepcopy
    # (respectively) of the original Simulation's __dict__.
    #
    # See OptionalSingleton for more information.
    def __new__(cls,*args,**kw):

        # simulate behavior of a parameter for register
        if 'register' in kw:
            register = kw['register']
        # this elif shouldn't be required, but is needed for
        # unpickling: OptionalSingleton's __getnewargs__ is called
        # with register as the first argument (I don't know how else
        # to set register on unpickling).
        elif len(args)==1:
            register = args[0]
        else:
            register = cls.register
            
        n = OptionalSingleton.__new__(cls,register)

        # CEBALERT: without removing references to the Sheets from
        # from instances of Slice, those instances of Slice and the
        # Sheets they refer to are never garbage collected. 
        #
        # This temporary implementation of cleanup code explicitly
        # removes references to Sheets from Slice instances for
        # typical simulations.
        #
        # 1: Better than having cleanup code here would be to make a
        # change to Slice, so that a Slice either doesn't have a
        # reference to a Sheet, or doesn't hold onto that reference.
        #
        # 2: Some cleanup code will always be required here in
        # Simulation.__new__: As well as removing references to Sheets
        # from Slices, it is also necessary to remove references to
        # sheets from Simulation's lists of EPs - otherwise the sheets
        # are not garbage collected and memory usage will go up every
        # time a new Simulation is created. This cleanup must be in
        # Simulation.__new__ so that it runs whenever a simulation is
        # created or unpickled (it can't be done e.g. in
        # load_snapshot).
        #
        # 3: this particular implementation assumes the only instances
        # of Slice are in ConnectionFields, which is true for our
        # simulations. (This won't matter when the slice cleanup
        # becomes unnecessary.)
        if hasattr(n,'_cleanup'):
            n._cleanup()
        # if we don't collect() here (exactly here - not in _cleanup,
        # and not later), gc seems to lose track of some objects and
        # there is still a (smaller) memory increase with every call
        # to load_snapshot()
        import gc
        gc.collect()

        return n

    # CEBALERT: see gc alert in __new__()
    def _cleanup(self):
        # will always be required: in case eps haven't been started
        # so are still in the list
        if hasattr(self,'eps_to_start'):
            self.eps_to_start[:]=[]
            
        if hasattr(self,'_event_processors'):
            for name,EP in self._event_processors.items():
                for c in EP.in_connections:
                    if hasattr(c,'_cleanup'):
                        c._cleanup()
                # will always be required
                self._event_processors[name]=None
                # (check when cleaning up existing mechanism for
                # adding sheets e.g. sim['x']=sheet could first set
                # sim['x'] to None if there is already a sheet with
                # name x...)
                
                
    # CEBALERT: if we're keeping this, should have a better name
    def convert_to_time_type(self,time):
        """
        Convert the supplied time to the Simulation's time_type.
        """
        # pylint: disable-msg=W0201
        if self.time_type_args:
            newtime = self.time_type(time,*self.time_type_args)
        else:
            newtime = self.time_type(time)
        return newtime


    # Note that __init__ can still be called after the
    # Simulation(register=True) instance has been created. E.g. with
    # Simulation.register is True,
    #   Simulation(name='A'); Simulation(name='B')
    #
    # would leave the single Simulation(register=True) instance with
    # name=='B'. This is because, as is usual in Python, __new__
    # creates an instance of a class, while __init__ is subsequently
    # given that instance (to initialize).

    def __init__(self,*args,**params):
        """
        Initialize a Simulation instance.
        """
        self._time = self.convert_to_time_type(0)

        param.Parameterized.__init__(self,**params)


        self._event_processors = {}

        if self.register:
            # Indicate that no specific name has been set
            self.name=params.get('name')
            # Set up debugging messages to include the simulator time
            param.parameterized.dbprint_prefix= \
               (lambda: "Time: "+self.timestr()+" ")

            param.Dynamic.time_fn = self.time


        self.events = [] # CB: consider collections.deque? (PEP 290)
        self._events_stack = []
        self.eps_to_start = []
        self.item_scale=1.0 # this variable determines the size of each item in a diagram
            
        # CB (this comment applies to SomeTimer!): make this a
        # parameter for documentation? Otherwise nobody will know
        # about being able to adjust step.
        #
        # we set step to 2 so that by default timing doesn't slow simulation too much. but
        # e.g. leaving it as None would result in info at 2% increments of requested run duration,
        # no matter what duration (0.005 or 5, etc).
        self.timer = SomeTimer(func=self.run,
                               simulation_time_fn=self.time)


    def __getitem__(self,item_name):
        """
        Return item_name if it exists as an EventProcessor in
        the Simulation. See objects().
        """
        if not isinstance(item_name,str):
            raise TypeError("Expected string (objects in the Simulation are indexed by name); %s is a %s"%(item_name,type(item_name)))
        try:
            return self.objects()[item_name]
        except KeyError:
            raise AttributeError("Simulation doesn't contain '"+item_name+"'.")


    # CEBALERT: should this at least give a warning when an existing
    # EP is replaced?
    def __setitem__(self,ep_name,ep):
        """
        Add ep to the simulation, setting its name to ep_name.
        ep must be an EventProcessor.

        If ep_name already exists in the simulation, ep overwrites
        the original object (as for a dictionary).

        Note: EventProcessors do not necessarily have to be added to
        the simulation to be used, but otherwise they will not receive the
        start() message.  Adding a node to the simulation also sets the
        backlink node.simulation, so that the node can enqueue events
        and read the simulation time.
        """
        if not isinstance(ep_name,str):
            raise TypeError("Expected string for item name (EPs in the Simulation are indexed by name).")

        if not isinstance(ep,EventProcessor):
            raise TypeError("Expected EventProcessor: objects in the Simulation must be EPs.")

        if ep in self._event_processors.values():
            self.warning("EventProcessor "+str(ep)+" () already exists in the simulation and will not be added.")
        else:
            ep.initialized=False
            ep.name=ep_name
            ep.initialized=True
            # deletes and overwrites any existing EP with the same name,
            # silently, as if a dictionary
            if ep.name in self._event_processors: del self[ep.name]
                
            self._event_processors[ep_name] = ep
            ep.simulation = self
            self.eps_to_start.append(ep)


    def __delitem__(self,ep_name):
        """
        Dictionary-style deletion of EPs from the simulation; see __delete_ep().

        Deletes EP from simulation, plus connections that come into it and
        connections that go out of it.
        """
        if not isinstance(ep_name,str):
            raise TypeError("Expected string for item name (EPs in the Simulation are indexed by name).")

        self.__delete_ep(ep_name)


    def __delete_ep(self,ep_name):
        """
        Remove the specified EventProcessor from the simulation, plus
        delete connections that come into it and connections that go from it.

        (Used by 'del sim[ep_name]' (as for a dictionary) to delete
        an event processor from the simulation.)
        """
        ep = self[ep_name]
        
        # remove from simulation list of eps
        del self._event_processors[ep_name]

        # remove out_conections that go to this ep
        for conn in ep.in_connections:
            conn.remove()

        # remove in_connections that come from this ep
        for conn in ep.out_connections:
            conn.remove()

    def __iter__(self):
        for obj in self.objects(): yield obj


    def __dir__(self):
        """
        Extend dir() to include simulation objects as well.  Useful
        for software that examines the list of possible objects, such
        as tab completion in IPython.
        """
        default_dir = dir(type(self)) + list(self.__dict__)
        return sorted(set(default_dir + self.objects().keys()))


    def __getattr__(self, name):
        """
        Provide a simpler attribute-like syntax for accessing objects
        within a Simulation (e.g. sim.obj1, for an EventProcessor
        "obj1" in sim).
        """
        if name=='_event_processors':
            raise AttributeError

        try:
            return self.objects()[name]
        except:
            raise AttributeError


    def time(self):
        """
        Return the current simulation time.
        
        If the time returned will be used in the computation of a
        floating point variable, it should be cast into a floating
        point number by float().
        """
        return self._time


    def timestr(self,specified_time=None):
        """
        Returns the specified time (or the current time, if none
        specified) formatted using time_printing_format, which allows
        users to control how much precision, etc. is used for time
        displays.
        """
        # CEBALERT: I doubt this gets all attributes. Does it get
        # properties (not that there are any right now)?
        all_vars = dict(self.get_param_values())
        all_vars.update(self.__dict__)
        if specified_time is not None:
            all_vars['_time']=specified_time
        timestr = self.time_printing_format % all_vars
        return timestr


    @property
    def timestr_prop(self):
        """
        A property that simply returns self.timestr(); useful for setting the
        interactive command-line prompt.
        """
        return self.timestr()


    def basename(self):
        """
        Return a string suitable for labeling an object created
        by the current simulation at the current time.  By default
        this is simply the name of the simulation + " " +
        the result from evaluating the time_printing_format parameter.
        """
        all_vars = dict(self.get_param_values())
        all_vars.update(self.__dict__)
        all_vars['timestr']=self.timestr()
        
        return self.basename_format % all_vars



    # Change current run() to _run(), and current run_and_time() to run()?

    # CEBALERT: need to simplify duration/until code. Hiding 'until' option
    # until it's fixed (presumably nobody's using it).
    def run_and_time(self,duration=Forever): # ,until=Forever):

        if duration==Forever:
            # CEBALERT: timing code not setup to handle indefinite durations
            # (e.g. 'Forever')
            self.run(duration)
            return
        else:
            self.timer.call_and_time(duration)


    def run(self,duration=Forever,until=Forever):
        """
        Process simulation events for the specified duration or until the specified time.

        Arguments:
        
          duration = length of simulation time to run. Default: run
          indefinitely while there are still events in the
          event_queue.

          until = maximum simulation time to simulate. Default: run
          indefinitely while there are still events in the event
          queue.

        If both duration and until are used, the one that is reached first will apply.


        Note that duration and until should be specified in a format suitable for
        conversion (coercion?) into the Simulation's _time_type.
        """
        # CEBALERT: need to do something about the global 'Forever';
        # what it should be probably varies with _time_type.

        # CEBALERT: calls to topo.sim.run() within topo should use a
        # string to specify the time rather than a float (since float
        # is not compatible with all number types).
        
        duration = self.convert_to_time_type(duration)
        until = self.convert_to_time_type(until)
        
        
        # CEBHACKALERT: If I do topo.sim.run(10), then topo.sim.run(until=3),
        # topo.sim._time returns to 3 (i.e. the simulation time can jump backwards).
        # JP: This because of the weird sim._time = stop_time line at the end of this method.
        # see my HACKALERT below.
        
        # Initialize any EPs that haven't been started yet
        #
        # Anything that manipulates the event stack in some way
        # (e.g. calls state_push() *must* ensure that this code has
        # been executed first (i.e. the code must call topo.sim.run(0)
        # before doing anything).  (Currently applies to
        # pattern_present(), Test Pattern's Present button, and
        # save_input_generators, but future code may need such calls
        # as well.)
        for e in self.eps_to_start:
            e.start()

        self.eps_to_start=[]

        
        # Complicated expression for min(time+duration,until)
        if duration == Forever:
            stop_time = until
        elif until == Forever:
            stop_time = self._time + duration
        else:
            stop_time = min(self._time+duration,until) 
            
        did_event = False

        while self.events and (stop_time == Forever or self._time <= stop_time):
            # Loop while there are events and it's not time to stop.
            
            if self.events[0].time < self._time:
                # Warn and then discard events scheduled *before* the current time
                self.warning('Discarding stale (unprocessed) event',repr(self.events[0]))
                self.events.pop(0)
                
            elif self.events[0].time > self._time:
                # Before moving on to the next time, do any processing
                # necessary for the current time.  This is necessary only
                # if some event has been delivered at the current time.

                if did_event:
                    did_event = False
                    #self.debug("Time to sleep; next event time: %s",self.timestr(self.events[0].time))
                    for ep in self._event_processors.values():
                        ep.process_current_time()
                    
                # Set the time to the frontmost event.  Bear in mind
                # that the front event may have been changed by the
                # .process_current_time() calls.
                if self.events[0].time > self._time:
                    self.sleep(self.events[0].time-self._time)
                
            else:
                # Pop and call the event at the head of the queue.
                event = self.events.pop(0)
                self.debug(lambda:"Delivering %s"%(event))
                event(self)
                did_event=True


        # The time needs updating if the events have not done it.
        #if self.events and self.events[0].time >= stop_time:

        # JPHACKALERT: This is weird.  It can cause time to go backwards,
        # (see CEBHACKALERT above).  Also, if the simulation runs out of
        # events before stop_time is reached, shouldn't that be reflected in
        # Simulation.time()?
        if stop_time != Forever :
            self._time = stop_time

    def sleep(self,delay):
        """
        Advance the simulator time by the specified amount.
        By default simply increments the _time value, but subclasses can
        override this method as they wish, e.g. to wait for an
        external real time clock to advance first.
        """
        self._time += self.convert_to_time_type(delay)

    def enqueue_event(self,event):
        """
        Enqueue an Event at an absolute simulation clock time.
        """
        assert isinstance(event,Event)

        if not self.events or event >= self.events[-1]:
            # The new event goes at the end of the event queue if there
            # isn't a queue right now, or if it's later than the last
            # event's time. 
            self.events.append(event)
        elif event < self.events[0]:
            # If it's earlier than the first item it goes at the beginning.
            self.events.insert(0,event)
        else:
            # Otherwise, it's inserted at the appropriate
            # position somewhere inside the event queue.
            # New events are enqueued after (right of) existing
            # events with the same time, i.e. 'simultaneous' events
            # are executed FIFO.
            bisect.insort_right(self.events,event)

    def schedule_command(self,time,command_string):
        """
        Add a command to execute in __main__.__dict__ at the
        specified time.

        The command should be a string.
        """
        event = CommandEvent(time=self.convert_to_time_type(time),command_string=command_string)
        self.enqueue_event(event)
        

    def state_push(self):
        """
        Save a copy of the current state of the simulation for later restoration.

        The saved copy includes all the events on the simulator stack
        (saved using event_push()).  Each EventProcessor is also asked
        to save its own state.  This operation is useful for testing
        something while being able to roll back to the original state.
        """
        self.event_push()
        for ep in self._event_processors.values():
            ep.state_push()

        param.Parameterized.state_push(self)


    def state_pop(self):
        """
        Pop the most recently saved state off the stack.

        See state_push() for more details.
        """
        self.event_pop()
        for ep in self._event_processors.values():
            ep.state_pop()

        param.Parameterized.state_pop(self)


    def event_push(self):
        """
        Save a copy of the events queue for later restoration.

        Same as state_push(), but does not ask EventProcessors to save
        their state.
        """
        # CBALERT: does it make more sense to put the original events onto the
        # stack, and replace self.events with the copies? Not sure this makes
        # any practical difference currently.
        self._events_stack.append((self._time,[copy(event) for event in self.events]))


    def event_pop(self):
        """
        Pop the most recently saved events queue off the stack.

        Same as state_pop(), but does not restore EventProcessors' state.
        """
        self._time, self.events = self._events_stack.pop()        


    def event_clear(self,event_type=EPConnectionEvent):
        """
        Clear out all scheduled events of the specified type.

        For instance, with event_type=EPConnectionEvent, this function can be used to ensure
        that no pending EPConnectionEvents will remain on the queue during some analysis 
        or measurement operation.  One will usually want to do a state_push before using this
        function, then clear out the events that should be deleted, do the measurement or 
        analysis, and then do state_pop to restore the original state.
        """
        events_temp = []
        for e in self.events:
            if not isinstance(e,event_type):
                events_temp = events_temp + [e]
        self.events = events_temp



    # Could just process src and dest in conn_params.  
    # Also could accept the connection already created, rather than
    # creating one.
    def connect(self,src,dest,connection_type=EPConnection,**conn_params):
        """
        Connect the src EventProcessor to the dest EventProcessor.

        The src and dest should be string names of existing EPs.
        Returns the connection that was created.  If the connection
        hasn't been given a name, it defaults to 'srcTodest'.
        """
        
        if 'name' not in conn_params:
            # Might want to have a way of altering the name if this one's
            # already in use. At the moment, an error is raised (correctly).
            conn_params['name'] = src+'To'+dest

        # Looks up src and dest in our dictionary of objects
        conn = connection_type(src=self[src],dest=self[dest],**conn_params)
        self[src]._src_connect(conn)
        self[dest]._dest_connect(conn)
        return conn
    

    def objects(self,baseclass=EventProcessor):
        """
        Return a dictionary of simulation objects having the specified
        base class.  All simulation objects have a base class of
        EventProcessor, and so the baseclass must be either
        EventProcessor or one of its subclasses.

        If there is a simulator called s, you can type e.g.
        s.objects().keys() to see a list of the names of all objects.
        """
        return dict([(ep_name,ep)
                     for (ep_name,ep) in self._event_processors.items()
                     if isinstance(ep,baseclass)])


    def connections(self):
        """Return a list of all unique connections to or from any object."""
        # The return value cannot be a dictionary like objects(),
        # because connection names are not guaranteed to be unique
        connlists =[o.in_connections + o.out_connections
                    for o in self.objects().values()]
        # Flatten one level
        conns=[]
        for cl in connlists:
            for c in cl:
                conns.append(c)
        return [c for c in set(conns)]


    def script_repr(self,imports=[],prefix="    "):
        """
        Return a nearly runnable script recreating this simulation.

        Needs some work to make the result truly runnable.

        Only scheduled commands that have not yet been executed are
        included, because executed commands are not kept around.
        """
        objs  = [o.script_repr(imports=imports) for o in
                 sorted(self.objects().values(), cmp=lambda x, y: cmp(x.name,y.name))]

        # CBENHANCEMENT: could allow user to plug in a sorting
        # function.  E.g. might want to compare conns based on name
        # then dest then src if lots of conns share the same name (so
        # the order is always the same).
        conns = [o.script_repr(imports=imports) for o in
                 sorted(self.connections(),      cmp=lambda x, y: cmp(x.name,y.name))]

        cmds  = [o.script_repr(imports=imports) for o in
                 sorted(sorted([e for e in self.events if isinstance(e,CommandEvent)],
                               cmp=lambda x, y: cmp(x.command_string,y.command_string)),
                        cmp=lambda x, y: cmp(x.time,y.time))]

        # CEBALERT: hack to support importing the time type since the
        # scheduled actions will have times printed using the
        # time_type.
        imports.append("from %s import %s"%(self.time_type.__module__,self.time_type.__name__))

        imps  = sorted(set(imports))

        vals  = [_simulation_path + "." + p + "=" + repr(getattr(self,p)) for p in
                 ["name","startup_commands"]
                 if getattr(self,p)]

        return "\n\n# Imports:\n\n"                +     '\n'.join(imps)  + \
               "\n\n\n"                            +   '\n\n'.join(vals)  + \
               '\n\n\n\n# Objects:\n\n'            + '\n\n\n'.join(objs)  + \
               '\n\n\n\n# Connections:\n\n'        + '\n\n\n'.join(conns) + \
               '\n\n\n\n# Scheduled commands:\n\n' +     '\n'.join(cmds)
    

    # Convenience function for use in graphical editors of the simulation
    def grid_layout(self,objgrid,xstart=100,xstep=150,ystart=100,ystep=150,item_scale=1.0):
        """
        Set the layout_location of simulation objects in a grid pattern.
    
        Takes a list of lists of simulation objects, or names of
        simulation objects, and positions them with layout_locations
        left-to-right, top-to-bottom, starting at (xstart,ystart) and
        advancing by xstep and ystep.
    
        The object None can be placed in the grid to skip a grid space.
        """
        
        self.item_scale=item_scale
        
        y = ystart
        for row in objgrid:
            x = xstart
            for obj in row:
                if obj:
                    if isinstance(obj,str):
                        self[obj].layout_location = x,y
                    else:
                        obj.layout_location = x,y
                x += xstep
            y += ystep



class RealTimeSimulation(Simulation):
    """
    A (quasi) real-time simulation object.

    This subclass of Simulation attempts to maintain a correspondence
    between simulation time and real time, as defined by the timescale
    parameter.  Real time simulation instances still maintain a
    nominal, discrete simulation time that determines the order of
    event delivery.

    At the beginning of each simulation time epoch, the simulation
    marks the actual wall clock time.  After event delivery for that
    epoch has ended, the simulation calculates the amount of
    computation time used for event processing, and executes a real
    sleep for the remainder of the epoch.  If the computation time for
    the epoch exceeded the real time, a warning is issued and
    processing proceeds immediately to the next simulation time epoch.


    RUN HOOKS

    The simulation includes as parameters two lists of functions/callables,
    run_start_hooks and run_stop_hooks, that will be called
    immediately before and after event processing during a call to
    .run().  This allows, for example, starting and stopping of
    real-time devices that might use resources while the simulation is
    not running.
    """
    
    timescale = param.Number(default=1.0,bounds=(0,None),doc="""
       The desired real length of one simulation time unit, in milliseconds.""")

    run_start_hooks = param.HookList(default=[],doc="""
       A list of callable objects to be called on entry to .run(),
       before any events are processed.""")
    
    
    run_stop_hooks = param.HookList(default=[],doc="""
       A list of callable objects to be called on exit from .run()
       after all events are processed.""") 

    def __init__(self,**params):
        super(RealTimeSimulation,self).__init__(**params)
        self._real_timestamp = 0.0
        
    def run(self,*args,**kw):
        for h in self.run_start_hooks:
            h()
        self._real_timestamp = self.real_time()
        super(RealTimeSimulation,self).run(*args,**kw)
        for h in self.run_stop_hooks:
            h()

    def real_time(self):
        return time.time() * 1000
    
    def sleep(self,delay):
        """
        Sleep for the number of real milliseconds seconds corresponding to the
        given delay, subtracting off the amount of time elapsed since the
        last sleep.
        """
        sleep_ms = delay*self.timescale-(self.real_time()-self._real_timestamp)

        if sleep_ms < 0:
            self.warning("Realtime fault. Sleep delay of %f requires realtime sleep of %.2f ms."
                         %(delay,sleep_ms))
        else:
            self.debug("sleeping. delay =",delay,"real delay =",sleep_ms,"ms.")
            time.sleep(sleep_ms/1000.0)
        self._real_timestamp = self.real_time()
        self._time += delay


        



"""
Transfer functions with more complex dependencies.

$Id: basic.py 10790 2009-11-21 17:51:33Z antolikjan $
"""
__version__='$Revision: 10790 $'

import copy

import numpy, numpy.random
from numpy import ones

import param

import topo
import topo.base.functionfamily

from topo.base.arrayutil import clip_lower,array_argmax
from topo.base.patterngenerator import PatternGenerator,Constant
from topo.base.boundingregion import BoundingBox
from topo.base.sheetcoords import SheetCoordinateSystem

from topo.transferfn import TransferFn,TransferFnWithState
from topo.pattern import Gaussian


# Not suitable for basic.py due to its dependence on patterns.
class PatternCombine(TransferFn):
    """
    Combine the supplied pattern with one generated using a
    PatternGenerator.

    Useful for operations like adding noise or masking out lesioned
    items or around the edges of non-rectangular shapes.
    """

    generator = param.ClassSelector(PatternGenerator,
        default=Constant(), doc="""
        Pattern to combine with the supplied matrix.""")

    operator = param.Parameter(numpy.multiply,precedence=0.98,doc="""
        Binary Numeric function used to combine the two patterns.

        Any binary Numeric array "ufunc" returning the same type of
        array as the operands and supporting the reduce operator is
        allowed here.  See topo.pattern.Composite.operator for more
        details.              
        """)
    
    def __call__(self,x):
        ###JABHACKALERT: Need to set it up to be independent of
        #density; right now only things like random numbers work
        #reasonably
        rows,cols = x.shape
        bb = BoundingBox(points=((0,0), (rows,cols)))
        generated_pattern = self.generator(bounds=bb,xdensity=1,ydensity=1).transpose()
        new_pattern = self.operator(x, generated_pattern)
        x *= 0.0
        x += new_pattern



# Not suitable for basic.py due to its dependence on patterns.
class KernelMax(TransferFn):
    """
    Replaces the given matrix with a kernel function centered around the maximum value.

    This operation is usually part of the Kohonen SOM algorithm, and
    approximates a series of lateral interactions resulting in a
    single activity bubble.

    The radius of the kernel (i.e. the surround) is specified by the
    parameter 'radius', which should be set before using __call__.
    The shape of the surround is determined by the
    neighborhood_kernel_generator, and can be any PatternGenerator
    instance, or any function accepting bounds, density, radius, and
    height to return a kernel matrix.
    """

    kernel_radius = param.Number(default=0.0,bounds=(0,None),doc="""
        Kernel radius in Sheet coordinates.""")
    
    neighborhood_kernel_generator = param.ClassSelector(PatternGenerator,
        default=Gaussian(x=0.0,y=0.0,aspect_ratio=1.0),
        doc="Neighborhood function")

    crop_radius_multiplier = param.Number(default=3.0,doc="""        
        Factor by which the radius should be multiplied, when deciding
        how far from the winner to keep evaluating the kernel.""")
    
    density=param.Number(1.0,bounds=(0,None),doc="""
        Density of the Sheet whose matrix we act on, for use
        in converting from matrix to Sheet coordinates.""")
    

    def __call__(self,x):
        rows,cols = x.shape
        radius = self.density*self.kernel_radius
        crop_radius = int(max(1.25,radius*self.crop_radius_multiplier))
        
        # find out the matrix coordinates of the winner
        wr,wc = array_argmax(x)
        
        # convert to sheet coordinates
        wy = rows-wr-1
        
        # Optimization: Calculate the bounding box around the winner
        # in which weights will be changed
        cmin = max(wc-crop_radius,  0)
        cmax = min(wc+crop_radius+1,cols)
        rmin = max(wr-crop_radius,  0)
        rmax = min(wr+crop_radius+1,rows)
        ymin = max(wy-crop_radius,  0)
        ymax = min(wy+crop_radius+1,rows)
        bb = BoundingBox(points=((cmin,ymin), (cmax,ymax)))
        
        # generate the kernel matrix and insert it into the correct
        # part of the output array
        kernel = self.neighborhood_kernel_generator(bounds=bb,xdensity=1,ydensity=1,
                                                    size=2*radius,x=wc+0.5,y=wy+0.5)
        x *= 0.0
        x[rmin:rmax,cmin:cmax] = kernel



class HalfRectify(TransferFn):
    """
    Transfer function that applies a half-wave rectification (clips at zero)
    """

    t_init = param.Number(default=0.0,doc="""
        The initial value of threshold at which output becomes non-zero..""")
    
    
    gain = param.Number(default=1.0,doc="""
        The neuronal gain""")
    
    randomized_init = param.Boolean(False,doc="""
        Whether to randomize the initial t parameter.""")
    
    noise_magnitude =  param.Number(default=0.1,doc="""
        The magnitude of the additive noise to apply to the t_init
        parameter at initialization.""")


    def __init__(self,**params):
        super(TransferFn,self).__init__(**params)
        self.first_call = True

    
    def __call__(self,x):
        if self.first_call:
            self.first_call = False
            if self.randomized_init:
                self.t = ones(x.shape, x.dtype.char) * self.t_init + \
		(topo.pattern.random.UniformRandom() \
                 (xdensity=x.shape[0],ydensity=x.shape[1])-0.5) * \
                 self.noise_magnitude*2
            else:
                self.t = ones(x.shape, x.dtype.char) * self.t_init
        
        x -= self.t
        clip_lower(x,0)
        x *= self.gain



class HomeostaticResponse(TransferFnWithState):
    """
    Adapts the parameters of a linear threshold function to maintain a
    constant desired average activity. Details can be found in the report 
    'Mechanisms for Stable and Robust Development of Orientation Maps
    and Receptive Fields', by Judith S. Law, Jan Antolik, and James A. 
    Bednar, 2011 (http://www.inf.ed.ac.uk/publications/report/1404.html).
    """

    t_init = param.Number(default=0.15,doc="""
        Initial value of the threshold value t.""")

    randomized_init = param.Boolean(False,doc="""
        Whether to randomize the initial t parameter.""")

    seed = param.Integer(default=42, doc="""
       Random seed used to control the initial randomized t values.""")

    target_activity = param.Number(default=0.024,doc="""
        The target average activity.""")

    linear_slope = param.Number(default=1.0,doc="""
        Slope of the linear portion above threshold.""")
        
    learning_rate = param.Number(default=0.01,doc="""
        Learning rate for homeostatic plasticity.""")
    
    smoothing = param.Number(default=0.991,doc="""
        Weighting of previous activity vs. current activity when 
        calculating the average activity.""")
        
    noise_magnitude =  param.Number(default=0.1,doc="""
        The magnitude of the additive noise to apply to the t_init
        parameter at initialization.""")
    
    period = param.Number(default=1.0, constant=True, doc="""
        How often the threshold should be adjusted.

        If the period is 0, the threshold is adjusted continuously, each 
        time this TransferFn is called.

        For nonzero periods, adjustments occur only the first time
        this TransferFn is called after topo.sim.time() reaches an
        integer multiple of the period.  
 
        For example, if period is 2.5 and the TransferFn is evaluated
        every 0.05 simulation time units, the threshold will be
        adjusted at times 2.55, 5.05, 7.55, etc.""")


    def __init__(self,**params):
        super(HomeostaticResponse,self).__init__(**params)
        self.first_call = True
        self.__current_state_stack=[]
        self.t=None     # To allow state_push at init
        self.y_avg=None # To allow state_push at init
        
        self._next_update_timestamp = topo.sim.time_type(topo.sim.time()+self.period)
        self._y_avg_prev = None
        self._x_prev = None


    def _initialize(self,x):
        self._x_prev = numpy.copy(x)
        self._y_avg_prev = ones(x.shape, x.dtype.char) * self.target_activity

        if self.randomized_init:
            self.t = ones(x.shape, x.dtype.char) * self.t_init + \
                (topo.pattern.random.UniformRandom( \
                    random_generator=numpy.random.RandomState(seed=self.seed)) \
                     (xdensity=x.shape[0],ydensity=x.shape[1]) \
                     -0.5)*self.noise_magnitude*2
        else:
            self.t = ones(x.shape, x.dtype.char) * self.t_init    
        self.y_avg = ones(x.shape, x.dtype.char) * self.target_activity


    def _apply_threshold(self,x):
        """Applies the piecewise-linear thresholding operation to the activity."""
        x -= self.t; clip_lower(x,0); x *= self.linear_slope


    def _update_threshold(self, prev_t, x, prev_avg, smoothing, learning_rate, target_activity):
        """
        Applies exponential smoothing to the given current activity and previous
        smoothed value following the equations given in the report cited above.
        """
        y_avg = (1.0-smoothing)*x + smoothing*prev_avg 
        t = prev_t + learning_rate * (y_avg - target_activity)
        return (y_avg, t)


    def __call__(self,x):
        """Initialises on the first call and then applies homeostasis."""
        if self.first_call: self._initialize(x); self.first_call = False

        if (topo.sim.time() > self._next_update_timestamp):
            self._next_update_timestamp += self.period
            # Using activity matrix and and smoothed activity from *previous* call.
            (self.y_avg, self.t) = self._update_threshold(self.t, self._x_prev, self._y_avg_prev, 
                                                          self.smoothing, self.learning_rate,
                                                          self.target_activity)
            self._y_avg_prev = self.y_avg   # Copy only if not in continuous mode

        self._apply_threshold(x)            # Apply the threshold only after it is updated
        self._x_prev[...,...] = x[...,...]  # Recording activity for the next periodic update


    def state_push(self):
        self.__current_state_stack.append((copy.copy(self.t),
                                           copy.copy(self.y_avg),
                                           copy.copy(self.first_call),
                                           copy.copy(self._next_update_timestamp),
                                           copy.copy(self._y_avg_prev),
                                           copy.copy(self._x_prev)))
        super(HomeostaticResponse, self).state_push()
        
    def state_pop(self):
        (self.t, self.y_avg, self.first_call, self._next_update_timestamp, 
        self._y_avg_prev, self._x_prev) = self.__current_state_stack.pop()
        super(HomeostaticResponse, self).state_pop()



class AttributeTrackingTF(TransferFnWithState):
    """
    Keeps track of attributes of a specified Parameterized over time, for analysis or plotting.

    Useful objects to track include sheets (e.g. "topo.sim['V1']"),
    projections ("topo.sim['V1'].projections['LateralInhibitory']"),
    or an output_function.  

    Any attribute whose value is a matrix the same size as the
    activity matrix can be tracked.  Only specified units within this
    matrix will be tracked.
    
    If no object is specified, this function will keep track of the
    incoming activity over time.

    The results are stored in a dictionary named 'values', as (time,
    value) pairs indexed by the parameter name and unit.  For
    instance, if the value of attribute 'x' is v for unit (0.0,0.0)
    at time t, values['x'][(0.0,0.0)]=(t,v).

    Updating of the tracked values can be disabled temporarily using
    the plastic parameter.
    """

    # ALERT: Need to make this read-only, because it can't be changed
    # after instantiation unless _object is also changed.  Or else
    # need to make _object update whenever object is changed and
    # _object has already been set.
    object = param.Parameter(default=None, doc="""
        Parameterized instance whose parameters will be tracked.

        If this parameter's value is a string, it will be evaluated first
        (by calling Python's eval() function).  This feature is designed to
        allow circular references, so that the TF can track the object that
        owns it, without causing problems for recursive traversal (as for
        script_repr()).""")
    # There may be some way to achieve the above without using eval(), which would be better.
    #JLALERT When using this function snapshots cannot be saved because of problem with eval()
    
    attrib_names = param.List(default=[], doc="""
        List of names of the function object's parameters that should be stored.""")
    
    units = param.List(default=[(0.0,0.0)], doc="""
        Sheet coordinates of the unit(s) for which parameter values will be stored.""")
    
    step = param.Number(default=1, doc="""
        How often to update the tracked values.

        For instance, step=1 means to update them every time this TF is
        called; step=2 means to update them every other time.""")

    coordframe = param.Parameter(default=None,doc="""
        SheetCoordinateSystem to use to convert the position into matrix coordinates.

        If this parameter's value is a string, it will be evaluated
        first(by calling Python's eval() function).  This feature is
        designed to allow circular references, so that the TF can
        track the object that owns it, without causing problems for
        recursive traversal (as for script_repr()).""")

    def __init__(self,**params):
        super(AttributeTrackingTF,self).__init__(**params)
        self.values={}
        self.n_step = 0
        self._object=None
        self._coordframe=None
        for p in self.attrib_names:
            self.values[p]={}
            for u in self.units:
                self.values[p][u]=[]
         
        
    def __call__(self,x):
    
        if self._object==None:
            if isinstance(self.object,str):
                self._object=eval(self.object)
            else:
                self._object=self.object
            
        if self._coordframe == None:
            if isinstance(self.coordframe,str) and isinstance(self._object,SheetCoordinateSystem):
                raise ValueError(str(self._object)+"is already a coordframe, no need to specify coordframe")
            elif isinstance(self._object,SheetCoordinateSystem):
                self._coordframe=self._object
            elif isinstance(self.coordframe,str):
                self._coordframe=eval(self.coordframe)
            else:
                raise ValueError("A coordinate frame (e.g. coordframe=topo.sim['V1']) must be specified in order to track"+str(self._object))
                
        #collect values on each appropriate step
        self.n_step += 1
        
        if self.n_step == self.step:
            self.n_step = 0
            if self.plastic:
                for p in self.attrib_names:
                    if p=="x":
                        value_matrix=x
                    else:
                        value_matrix= getattr(self._object, p)
                        
                    for u in self.units:
                        mat_u=self._coordframe.sheet2matrixidx(u[0],u[1])
                        self.values[p][u].append((topo.sim.time(),value_matrix[mat_u]))
          


__all__ = list(set([k for k,v in locals().items() if isinstance(v,type) and issubclass(v,TransferFn)]))
__all__.remove("TransferFn")
__all__.remove("TransferFnWithState")




#!/usr/bin/env python

import sys
from distutils.core import setup
import glob

examples = glob.glob('examples/*.ty')
models = glob.glob('models/*.ty')
scripts = ['topographica']

### TOPOGRAPHICA DEPENDENCIES ########################################
required = {'pillowfight':">=0.2",
            'numpy':">=1.0,<1.9",
            'param':">=1.0,<1.2",
            'paramtk':"==0.8.1",
            'imagen':"==1.0.1"}

optional = {'gmpy':'>=1.0', 
            'matplotlib':'>=0.8',
            'scipy':'>=0.5',
            'ipython':">=0.7"}
            
# for pip/easy_install (just go for the basics until easy_install works better!)
packages_to_install = [required]

packages_to_state = [required]
######################################################################

setup_args = {}


if 'setuptools' in sys.modules:
    # support easy_install without depending on setuptools
    install_requires = []
    for package_list in packages_to_install:
        install_requires+=["%s%s"%(package,version) for package,version in package_list.items()]
    setup_args['install_requires']=install_requires
    setup_args['dependency_links']=["http://pypi.python.org/simple/"]
    setup_args['zip_safe']=False

for package_list in packages_to_state:
    requires = []
    requires+=["%s (%s)"%(package,version) for package,version in package_list.items()]
    setup_args['requires']=requires


if 'bdist_wininst' in sys.argv:
    scripts.append('platform/windows/windows_postinstall.py')
    # CEBALERT: how else to get the ico into a place the postinstall script can find?!
    scripts.append("platform/windows/topographica.ico")

TRAINSCRIPTS = [
    "hierarchical.ty",
    "lissom_or.ty",
    "lissom_oo_or.ty",
    "som_retinotopy.ty",
    "sullivan_neurocomputing04.ty",
    "lissom.ty",
    "lissom_fsa.ty",
    "gcal.ty"
    ]


_topographica_devs='Topographica Developers'
_topographica_devs_email='developers[at]topographica[dot]org' 


setup_args.update(dict(
    name='topographica',

    # CEBALERT: we need one single place with the version number!  
    #
    # This release number is usually 1 higher than the Makefile's,
    # except when the Makefile is updated just prior to release.  This
    # is because DEBs etc being built between releases have a release
    # number of the next release combined with the svn revision
    # number, to match DEB convention. (Whereas for svn copy we are
    # using the opposite convention...)
    version='0.9.8-1',

    description='A general-purpose neural simulator focusing on topographic maps.',

    long_description="""
`Topographica`_ is a software package for computational modeling of
neural maps. The goal is to help researchers understand brain function
at the level of the topographic maps that make up sensory and motor
systems.

Please see http://topographica.org/ for more information.

Installation
============

Please see http://topographica.org/Downloads for more complete
instructions; a brief summary for installation into an existing
Python environment is given here.

If you have `pip`_, you can use one of
those to install Topographica and its dependencies automatically
(e.g. ``pip install topographica``).

Alternatively, you can download and unpack the archive below, and then
install Topographica with a command like ``python setup.py install``
(e.g. ``sudo python setup.py install`` for a site-wide installation,
or ``python setup.py install --user`` to install into
``~/.local``). You will need to install at least `NumPy`_, `Pillow`_ (or `PIL`_), 
`Param`_, `ParamTk`_, and `ImaGen`_, before running Topographica. We
also recommend that you install `MatPlotLib`_ so you can access all
Topographica's plots, as well as `GmPy`_ and Weave (available as part
of `SciPy`_) for optimum performance.

.. _Topographica:
   http://topographica.org/Home/
.. _NumPy: 
   http://pypi.python.org/pypi/numpy
.. _Gmpy: 
   http://pypi.python.org/pypi/gmpy
.. _SciPy: 
   http://pypi.python.org/pypi/scipy
.. _MatPlotLib: 
   http://pypi.python.org/pypi/matplotlib
.. _Pillow: 
   http://pypi.python.org/pypi/pillow
.. _PIL: 
   http://pypi.python.org/pypi/PIL
.. _pip:
   http://pip.openplans.org/
""",

    author= _topographica_devs,
    author_email= _topographica_devs_email,
    maintainer= _topographica_devs,
    maintainer_email= _topographica_devs_email,
    platforms=['Windows', 'Mac OS X', 'Linux'],
    license='BSD',
#    download_url='http://sourceforge.net/projects/topographica/files/',
    url='http://topographica.org/',

    classifiers = [
        "License :: OSI Approved :: BSD License",
# (until packaging tested)
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Operating System :: OS Independent",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "Natural Language :: English",
        "Topic :: Education",
        "Topic :: Scientific/Engineering"],

    # CEBALERT: do I have to list these? if I do, can I generate the list automatically?
    packages=['topo',
              'topo.analysis',
              'topo.base',
              'topo.command',
              'topo.coordmapper',
              'topo.ep',
              'topo.learningfn',
              'topo.misc',
              'topo.plotting',
              'topo.projection',
              'topo.responsefn',
              'topo.sheet',
              'topo.tests',
              'topo.tkgui',
              'topo.transferfn'],

    package_data={
        # CB: These things are not data, but there's currently no
        # other mechanism in distutils/setuptools.
        '': ['topographica'],
        'topo.tkgui': ['icons/*.*'],
        'topo.command':['*.png','*.pdf'],
        'topo.tests':['*.txt','*.jpg','*.pgm']+[s+"_DATA" for s in TRAINSCRIPTS]+['*.data']+['*.typ']+['*.pickle']},

    data_files=[('share/topographica/examples',examples),
                ('share/topographica/models',models)],

    scripts = scripts))


if __name__=="__main__":
    setup(**setup_args)



"""
Test for the Plot class.

$Id$
"""


import unittest
from pprint import pprint
from topo.plotting import plot
from topo.base.sheet import *
from topo.plotting.bitmap import RGBBitmap, HSVBitmap
#from testsheetview import ImageGenerator

SHOW_PLOTS = False


### JC: My new imports
from topo.plotting.plot import TemplatePlot, make_template_plot
import numpy.oldnumeric as Numeric
from numpy.oldnumeric import zeros, divide, Float, ones,reshape,array
from topo.base.boundingregion import BoundingBox
from topo.base.sheetview import SheetView
import numpy.oldnumeric.mlab as MLab
import numpy.oldnumeric.random_array as RandomArray

import param

from random import random


### This function is defined here, where it might be useful for testing
### Plot
def matrix_hsv_to_rgb(hMapArray,sMapArray,vMapArray):
    """
    First matrix sets the Hue (Color).
    Second marix sets the Sauration (How much color)
    Third matrix sets the Value (How bright the pixel will be)

    The three input matrices should all be the same size, and have
    been normalized to 1.  There should be no side-effects on the
    original input matrices.
    """
    
    shape = hMapArray.shape
    rmat = array(hMapArray,Float)
    gmat = array(sMapArray,Float)
    bmat = array(vMapArray,Float)
    
    ## This code should never be seen.  It means that calling code did
    ## not take the precaution of clipping the input matrices.
    if max(rmat.ravel()) > 1 or max(gmat.ravel()) > 1 or max(bmat.ravel()) > 1:
        param.Parameterized().warning('HSVBitmap inputs exceed 1. Clipping to 1.0')
        if max(rmat.ravel()) > 0: rmat = clip(rmat,0.0,1.0)
        if max(gmat.ravel()) > 0: gmat = clip(gmat,0.0,1.0)
        if max(bmat.ravel()) > 0: bmat = clip(bmat,0.0,1.0)

    # List comprehensions were not used because they were slower.
    for j in range(shape[0]):
        for i in range(shape[1]):
            rgb = hsv_to_rgb(rmat[j,i],gmat[j,i],bmat[j,i])
            rmat[j,i] = rgb[0]
            gmat[j,i] = rgb[1]
            bmat[j,i] = rgb[2]
                
    return (rmat, gmat, bmat)



### JCALERT !he file should be rewritten according to new changes in Plot.

class TestPlot(unittest.TestCase):

    def setUp(self):

        ### Simple case: we only pass a dictionnary to Plot()
        ### that does not belong to a Sheet:
        self.view_dict = {}

        ### SheetView1:
        ### Find a way to assign randomly the matrix.
        self.matrix1 = zeros((10,10),Float) + RandomArray.random((10,10))
        self.bounds1 = BoundingBox(points=((-0.5,-0.5),(0.5,0.5)))
        self.sheet_view1 = SheetView((self.matrix1,self.bounds1),
                                      src_name='TestInputParam')
        self.key1 = 'sv1'
        self.view_dict[self.key1] = self.sheet_view1

        ### SheetView2:
        ### Find a way to assign randomly the matrix.
        self.matrix2 = zeros((10,10),Float) + 0.3
        self.bounds2 = BoundingBox(points=((-0.5,-0.5),(0.5,0.5)))
        self.sheet_view2 = SheetView((self.matrix2,self.bounds2),
                                      src_name='TestInputParam')
        self.key2 = ('sv2',0,10)
        self.view_dict[self.key2] = self.sheet_view2

        ### SheetView3:
        ### Find a way to assign randomly the matrix.
        self.matrix3 = zeros((10,10),Float) + RandomArray.random((10,10))
        self.bounds3 = BoundingBox(points=((-0.5,-0.5),(0.5,0.5)))
        self.sheet_view3 = SheetView((self.matrix3,self.bounds3),
                                      src_name='TestInputParam')
        self.key3 = ('sv3',0,'hello',(10,0))
        self.view_dict[self.key3] = self.sheet_view3

        ### SheetView4: for testing clipping + different bounding box
        ### Find a way to assign randomly the matrix.
        self.matrix4 = zeros((10,10),Float) + 1.6
        self.bounds4 = BoundingBox(points=((-0.7,-0.7),(0.7,0.7)))
        self.sheet_view4 = SheetView((self.matrix4,self.bounds4),
                                      src_name='TestInputParam')
        self.key4 = 'sv4'
        self.view_dict[self.key4] = self.sheet_view4

        ### JCALERT! for the moment we can only pass a triple when creating plot
        ### adding more sheetView to test when plot will be fixed for accepting
        ### as much as you want.

        # plot0: empty plot + no sheetviewdict passed: error or empty plot?
        ### JCALERT! It has to be fixed what to do in this case in plot..
        ### disabled test for the moment.
        #self.plot0 = Plot((None,None,None),None,name='plot0')
        ### CATCH EXCEPTION

        plot_channels1 = {'Strength':None,'Hue':None,'Confidence':None}
        # plot1: empty plot 
        self.plot1 = make_template_plot(plot_channels1,self.view_dict,density=10.0,name='plot1')
        
        plot_channels2 = {'Strength':self.key1,'Hue':None,'Confidence':None}                       
        # plot2: sheetView 1, no normalize, no clipping
        self.plot2 = make_template_plot(plot_channels2,self.view_dict,density=10.0,name='plot2')
 
        plot_channels3 = {'Strength':self.key1,'Hue':self.key2,'Confidence':None}
        # plot3: sheetView 1+2, no normalize, no clipping
        self.plot3 = make_template_plot(plot_channels3,self.view_dict,density=10.0,name='plot3')

        plot_channels4 = {'Strength':self.key1,'Hue':self.key2,'Confidence':self.key3}
        # plot4: sheetView 1+2+3, no normalize , no clipping 
        self.plot4 = make_template_plot(plot_channels4,self.view_dict,density=10.0,name='plot4')

        plot_channels5 = {'Strength':self.key1,'Hue':None,'Confidence':self.key3}
        # plot5: sheetView 1+3, no normalize, no clipping
        self.plot5 = make_template_plot(plot_channels5,self.view_dict,density=10.0,name='plot5')

        plot_channels6 = {'Strength':None,'Hue':self.key2,'Confidence':self.key3}
        # plot6: sheetView 2+3, no normalize , no clipping 
        self.plot6 = make_template_plot(plot_channels6,self.view_dict,density=10.0,name='plot6')

        plot_channels7 = {'Strength':self.key4,'Hue':self.key2,'Confidence':self.key3}
        # plot7: sheetView 1+2+3, no normalize , clipping 
        self.plot7 = make_template_plot(plot_channels7,self.view_dict,density=10.0,name='plot7')

        plot_channels8 = {'Strength':self.key1,'Hue':self.key2,'Confidence':self.key3}
        # plot8: sheetView 1+2+3, normalize , no clipping 
        self.plot8 = make_template_plot(plot_channels8,self.view_dict,density=10.0,normalize=True,name='plot8')

        ### JCALERT! FOR THE MOMENT I TAKE THE DEFAULT FOR NORMALIZE.
        ### WE WILL SEE IF IT REMAINS IN PLOT FIRST.

        ### also makes a sheet to test realease_sheetviews

        self.sheet = Sheet()
        self.sheet.sheet_views[self.key1]=self.sheet_view1
        self.sheet.sheet_views[self.key2]=self.sheet_view2
        self.sheet.sheet_views[self.key3]=self.sheet_view3
        self.sheet.sheet_views[self.key4]=self.sheet_view4

        plot_channels9 = {'Strength':self.key1,'Hue':self.key2,'Confidence':self.key3}
        self.plot9 = make_template_plot(plot_channels9,self.sheet.sheet_views,density=10.0,name='plot9')

        
        

    def test_plot(self):
        pass
    
#       ### JCALERT! make a test for plot0

#       # plot 1
#       test = None
#       self.assertEqual(self.plot1,test)

#       # plot 2
#       sat = zeros((10,10),Float) 
#       hue = zeros((10,10),Float)
#       val = self.matrix1

#       test = matrix_hsv_to_rgb(hue,sat,val)
#       for each1,each2 in zip(self.plot2.rgb_matrices,test):
#           for each3,each4 in zip(each1.ravel(),each2.ravel()):
#               self.assertAlmostEqual(each3,each4)

#       # plot 3
#       sat = ones((10,10),Float) 
#       hue = zeros((10,10),Float) + 0.3
#       val = self.matrix1

#       test = matrix_hsv_to_rgb(hue,sat,val)
#       for each1,each2 in zip(self.plot3.rgb_matrices,test):
#           for each3,each4 in zip(each1.ravel(),each2.ravel()):
#               self.assertAlmostEqual(each3,each4)  

#       # plot 4
#       sat = self.matrix3 
#       hue = zeros((10,10),Float) + 0.3
#       val = self.matrix1

#       test = matrix_hsv_to_rgb(hue,sat,val)
#       for each1,each2 in zip(self.plot4.rgb_matrices,test):
#           for each3,each4 in zip(each1.ravel(),each2.ravel()):
#               self.assertAlmostEqual(each3,each4)  

#       # plot 5
#       sat = zeros((10,10),Float) 
#       hue = zeros((10,10),Float) 
#       val = self.matrix1

#       test = matrix_hsv_to_rgb(hue,sat,val)
#       for each1,each2 in zip(self.plot5.rgb_matrices,test):
#           for each3,each4 in zip(each1.ravel(),each2.ravel()):
#               self.assertAlmostEqual(each3,each4)

#       # plot 6
#       sat = self.matrix3 
#       hue = zeros((10,10),Float) + 0.3 
#       val = ones((10,10),Float) 

#       test = matrix_hsv_to_rgb(hue,sat,val)
#       for each1,each2 in zip(self.plot6.rgb_matrices,test):
#           for each3,each4 in zip(each1.ravel(),each2.ravel()):
#               self.assertAlmostEqual(each3,each4)  
    
#       # plot 7
#       sat = self.matrix3 
#       hue = zeros((10,10),Float) + 0.3 
#       val = self.matrix4

#         val = MLab.clip(val,0.0,1.0)
        
#       test = matrix_hsv_to_rgb(hue,sat,val)
#       for each1,each2 in zip(self.plot7.rgb_matrices,test):
#           for each3,each4 in zip(each1.ravel(),each2.ravel()):
#               self.assertAlmostEqual(each3,each4)
        

#       # plot 8
#       sat = self.matrix3 
#       hue = zeros((10,10),Float) + 0.3 
#       val = self.matrix1

#       val = divide(val,float(max(val.ravel())))
        
#       test = matrix_hsv_to_rgb(hue,sat,val)

#       for each1,each2 in zip(self.plot8.rgb_matrices,test):
#           for each3,each4 in zip(each1.ravel(),each2.ravel()):
#               self.assertAlmostEqual(each3,each4)




#       # plot 9
#       sat = self.matrix3 
#       hue = zeros((10,10),Float) + 0.3 
#       val = self.matrix1
        
#       test = matrix_hsv_to_rgb(hue,sat,val)
#       for each1,each2 in zip(self.plot9.rgb_matrices,test):
#           for each3,each4 in zip(each1.ravel(),each2.ravel()):
#               self.assertAlmostEqual(each3,each4)  

# #### Think about doing a plot test using sheet_dict and a sheet?
# ### Ask Jim if it is really necessary...

#     def test_release_sheetviews(self):

#       self.plot9.release_sheetviews()

#       test=self.sheet.sheet_views.get(self.key1,None)
#       self.assertEqual(test,None)
#       test=self.sheet.sheet_views.get(self.key2,None)
#       self.assertEqual(test,None)
#       test=self.sheet.sheet_views.get(self.key3,None)
#       self.assertEqual(test,None)
#       test=self.sheet.sheet_views.get(self.key4,None)
#       self.assertEqual(test,self.sheet_view4)


#     def test_matrix_hsv_to_rgb(self):
#         a = [j for i in range(256) for j in range(256)]
#         b = [i for i in range(256) for j in range(256)]
#         c = [max(i,j) for i in range(256) for j in range(256)]
#         a = Numeric.reshape(a,(256,256)) / 255.0
#         b = Numeric.reshape(b,(256,256)) / 255.0
#         c = Numeric.reshape(c,(256,256)) / 255.0
#         (h,s,v) = matrix_hsv_to_rgb(a,b,c)
#         rgb = RGBMap(h,s,v)
#         # rgb.show()

#     def test_matrix_hsv_to_rgb2(self):
#         h = Numeric.array([[0.0,0.0],[0.0,0.0]])
#         s = Numeric.array([[0.0,0.0],[0.0,0.0]])
#         v = Numeric.array([[0.5,0.5],[0.5,0.5]])
#         h_orig = Numeric.array(h)
#         s_orig = Numeric.array(s)
#         v_orig = Numeric.array(v)
#         r,g,b = matrix_hsv_to_rgb(h,s,v)
#         rgb_target = Numeric.array([[0.5,0.5],[0.5,0.5]])
#         self.assertEqual(h,h_orig)
#         self.assertEqual(s,s_orig)
#         self.assertEqual(v,v_orig)
    

### JC: THIS CODE IS LEFT TEMPORARY IN CASE IT IS OF ANY USE IN NEAR FUTURE
    
#         x = plot.Plot(('Activity',None,None),plot.COLORMAP,self.s2)
#         for o in dir():
#             # pprint(o)
#             if isinstance(o,plot.Plot):
#                 o.warning('Found ', o.name)

#         input = ImageGenerator(filename='topo/tests/testsheetview.ppm',
#                          density=100,
#                          bounds=BoundingBox(points=((-0.8,-0.8),(0.8,0.8))))
#         sv = input.sheet_view('Activity')

#         # Defined sheetview in the R channel
#         plot1 = plot.Plot((None,None,sv),plot.COLORMAP)
#         p_tuple = plot1.plot()
#         (r, g, b) = p_tuple.matrices
#         map = RGBMap(r,g,b)
#         if SHOW_PLOTS: map.show()


#     def test_HSV_plot(self):
#         input = ImageGenerator(filename='topo/tests/testsheetview.ppm',
#                          density=100,
#                          bounds=BoundingBox(points=((-0.8,-0.8),(0.8,0.8))))
#         sv = input.sheet_view('Activity')

#         # Defined sheetview in the R channel
#         plot1 = plot.Plot((sv,sv,sv),plot.HSV)
#         p_tuple = plot1.plot()
#         (r, g, b) = p_tuple.matrices
#         map = HSVMap(r,g,b)
#         if SHOW_PLOTS: map.show()

#     def test_plottemplate(self):
#         pt = plot.PlotTemplate()
#         pt = plot.PlotTemplate({'Strength'   : None,
#                                 'Hue'        : 'HueP',
#                                 'Confidence' : None})
#         pt = plot.PlotTemplate(channels={'Strength'   : None,
#                                          'Hue'        : 'HueP',
#                                          'Confidence' : None})



suite = unittest.TestSuite()
suite.addTest(unittest.makeSuite(TestPlot))
if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite)



"""
KeyedList sorted dictionary class.

$Id$
"""


# CEBALERT: Do most uses of KeyedList look like they would be better
# served by something that's really a dictionary, but which maintains
# order (rather than something that's really a list, but allows some
# dictionary-style access)? If so, we could use one of the many online
# ordered dictionaries instead.
# http://cheeseshop.python.org/pypi/Ordered%20Dictionary/0.2.2
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/496761
# http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/107747 
#
# Sometimes having a KeyedList is confusing, because you can't use
# natural syntax such as "x in y" when y is a KeyedList. 
#
# JB: Sure, we could use one of those instead; no good reason to
# maintain our own not-quite-full-featured version.


class KeyedList(list):
    """
    Extends the built-in type 'list' to support dictionary-like
    access using [].  The internal representation is as an ordinary
    list of (key,value) pairs, not a hash table like an ordinary
    dictionary, so that the elements will remain ordered.

    Note: Not all list operations will work as expected, because
    [] does not return the name tuple.

    Redefined functions::
    
        __getitem__ ([,])
        __setitem__ ([,])
        append  --  Now takes a tuple (key,value) so that value
                    can be later accessed by [key].
    
    New functions modeled from dictionaries::
    
        get
        set
        has_key
        keys
        items
        update

    Key values are not allowed to be None, because None is a
    default return value for get() when there is no object
    by that name.
    """

    def __getitem__(self,key):
        """The bracket [] accessor."""
        for (name,value) in self:
            if name == key:
                return value
            
        # Though not often useful for this class, the list interface
        # provides for access by an integer key:
        if isinstance(key,int):
            for (index,(name,value)) in enumerate(self):
                if index == key:
                    return value

        raise KeyError(key)


    def __setitem__(self,k,v):
        """
        The bracket [] mutator.
        Overwrite value if key already exists, otherwise append.
        """
        return self.set(k,v)


    def append(self, (key, value)):
        """
        Append a new object to the end of the existing list.

        Accepts a 2-tuple (key, value).

        Strictly speaking, this operation did not need to be redefined
        in this subclass, but by forcing the tuple in the function
        parameters, we may be able to catch an erroneous assignment.
        """
        super(KeyedList,self).append(tuple((key,value)))


    def get(self, key, default=None):
        """
        Get the value with the specified key.
        
        Returns None if no value with that key exists.
        """
        for (name,value) in self:
            if name == key:
                return value
            
        return default


    def set(self, key, value):
        """
        If the key already exists in the list, change the entry.
        Otherwise append the new (key,value) to the end of the list.
        """
        for (k,v) in self:
            if k == key:
                i = self.index((k,v))
                self.pop(i)
                self.insert(i,(key, value))
                return True
        self.append((key, value))
        return True


    def has_key(self,key):
        """Return True iff key is found in the ordered list."""
        for (name,value) in self:
            if name == key:
                return True
        return False


    def items(self):
        """
        Provide the item function supported by dictionaries.  
        A keyed list already is stored in this format, so just returns
        the actual underlying list.
        """
        return list(self)


    def keys(self):
        """A copy of the list of keys."""
        return [k for (k,v) in self.items()]


    def values(self):
        """A copy of the list of values."""
        return [v for (k,v) in self.items()]


    def update(self,b):
        """Updates (and overwrites) key/value pairs from b."""
        for (k,v) in b.items():
            self.set(k,v)



"""
Functional GUI tests: run_basic() and run_detailed().

$Id$
"""

import copy

from numpy import array
from numpy.testing import assert_array_equal


import topo
assert hasattr(topo,'guimain'), "GUI must be running."

import topo.tests.functionaltest as ft
from topo.tests.utils import assert_array_not_equal

# typing shortcut
g = topo.guimain

def run_basic():
    """Check that the windows all open ok (i.e. is GUI functioning?)."""
    _initialize()

    s = 'Simulation'
    p = 'Plots'
    
    menu_paths = [ (s,'Test Pattern'),
                   (s,'Model Editor'),
                   (p,'Activity'),
                   (p,'Connection Fields'),
                   (p,'Projection'),
                   (p,'Projection Activity'),
                   (p,'Preference Maps','Orientation Preference'),
                   (p,'Tuning Curves','Orientation Tuning') ]
    
    return ft.run([_menu_item_fn(*x) for x in menu_paths],"Running basic GUI tests...")



def run_detailed():
    """Test that more complex GUI actions are working."""
    _initialize()
    tests = [test_cf_coords,test_test_pattern,
             test_projection,test_orientation_tuning] # and so on...
    return ft.run(tests,"Running detailed GUI tests...")



######################################################################
# DETAILED TESTS
######################################################################

def test_cf_coords():
    """Check that ConnectionFields window opens with specified coords."""
    cf = g['Plots']['Connection Fields'](x=0.125,y=0.250)
    assert cf.x==0.125
    assert cf.y==0.250
    

def test_test_pattern():
    """Check that test pattern window is working."""
    tp = g['Simulation']['Test Pattern']()
    act = g['Plots']['Activity']()

    tp.gui_set_param('edit_sheet','GS')

    ## Test for state_push bug (simulation not run() before Present pressed)
    assert len(topo.sim.eps_to_start)>0, "test must be run before simulation is run()"
    from topo.pattern import Gaussian
    from topo import numbergen
    topo.sim['GS'].set_input_generator(Gaussian(x=numbergen.UniformRandom()))
    tp.Present()
    topo.sim.run(1)
    act1 = copy.deepcopy(topo.sim['GS'].activity)
    topo.sim.run(2)
    assert_array_not_equal(topo.sim['GS'].activity,act1,"GeneratorSheet no longer generating patterns")
    ##
    
    tp.gui_set_param('pattern_generator','TwoRectangles')
    from topo.pattern import TwoRectangles
    assert isinstance(tp.pattern_generator,TwoRectangles), "Pattern generator did not change."

    preview = _get_named_plot('GS',tp.plotgroup.plots).view_dict['Activity'].view()[0]
    two_rectangles = array([[0.,1],[1.,0.]])
    assert_array_equal(preview,two_rectangles,"Incorrect pattern in preview plot.")


    tp.Present()
    gs_view = _get_named_plot('GS',act.plotgroup.plots).view_dict['Activity']
    assert gs_view.src_name=='GS'
    gs_plot_array = gs_view.view()[0]
    assert_array_equal(gs_plot_array,two_rectangles,"Incorrect pattern in activity plot after Present.")


    tp.params_frame.gui_set_param('scale',0.5)
    preview = _get_named_plot('GS',tp.plotgroup.plots).view_dict['Activity'].view()[0]
    assert_array_equal(preview,0.5*two_rectangles,"Changing pattern parameters did not update preview.")

    
    ### Defaults button

    # first change several more parameters
    initial_preview = tp.plotgroup.plots[0].view_dict['Activity'].view()[0]
    
    new_param_values = [#('output_fns','Sigmoid'),
                        ('scale','2')]

    for name,value in new_param_values:
        tp.params_frame.gui_set_param(name,value)

    changed_preview = _get_named_plot('GS',tp.plotgroup.plots).view_dict['Activity'].view()[0]
    # and check the preview did change
    try:
        assert_array_equal(changed_preview,initial_preview)
    except AssertionError:
        pass
    else:
        raise AssertionError("Test pattern didn't change.")
    
    # test that preview display is correct
    tp.params_frame.Defaults()
    preview = _get_named_plot('GS',tp.plotgroup.plots).view_dict['Activity'].view()[0]
    assert_array_equal(preview,two_rectangles,"Defaults button failed to revert params to default values.")

    # CB: still need to test duration, learning, etc



def test_projection():
    """Check the Projection window."""
    
    p = g['Plots']['Projection']()
    p.gui_set_param('sheet','S')
    p.gui_set_param('projection','GSToS')

    p.gui_set_param('sheet','S2')
    p.gui_set_param('projection','GS2ToS2')
    p.gui_set_param('projection','GSToS2')
    

def test_orientation_tuning():
    """Check that orientation tuning plot works."""

    p = g['Plots']['Tuning Curves']['Orientation Tuning']()
    from topo.command.pylabplot import measure_or_tuning
    p.pre_plot_hooks = [measure_or_tuning.instance(num_phase=1,num_orientation=1,curve_parameters=[{'contrast':30}])]
    p.Refresh()


######################################################################
# UTILITY FUNCTIONS
######################################################################

# make these particular tests simpler

def _initialize():
    """Make a simple simulation."""
    from topo.base.simulation import Simulation
    from topo.base.cf import CFSheet,CFProjection
    from topo.sheet import GeneratorSheet

    sim=Simulation(register=True,name="test pattern tester")
    sim['GS']=GeneratorSheet(nominal_density=2)
    sim['GS2']=GeneratorSheet(nominal_density=2)
    sim['S'] = CFSheet(nominal_density=2)
    sim['S2'] = CFSheet(nominal_density=2)
    sim.connect('GS','S',connection_type=CFProjection,delay=0.05)
    sim.connect('GS','S2',connection_type=CFProjection,delay=0.05)
    sim.connect('GS2','S2',connection_type=CFProjection,delay=0.05)
    global g
    g=topo.guimain


def _menu_item_fn(*clicks):
    """Return a wrapper round topo.guimain[click1][click2]...[clickN], with __doc__ set
    to the menu path."""
    menu_path = 'topo.guimain'

    for click in clicks:
        menu_path+='["%s"]'%click

    menu_item = eval(menu_path)

    def test_menu_item():
        menu_item()

    test_menu_item.__doc__ = "%s"%menu_path[12::]

    return test_menu_item



def _get_named_plot(name,plots):

    for plot in plots:
        if plot.plot_src_name==name:
            return plot

    assert False
    



"""
Projection-level response functions.

For CFProjections, these function objects compute a response matrix
when given an input pattern and a set of ConnectionField objects.

$Id$
"""

from numpy import sum,exp,zeros,ravel
from numpy.oldnumeric import Float

import param

from topo.base.cf import CFPResponseFn
from topo.base.functionfamily import ResponseFn,DotProduct
from topo.base.arrayutil import L2norm

# Imported here so that all ResponseFns will be in the same package
from topo.base.cf import CFPRF_Plugin  # pyflakes:ignore (API import)


# CEBERRORALERT: doesn't use iterator, so ignores
# sheet mask!
class CFPRF_EuclideanDistance(CFPResponseFn):
    """
    Euclidean-distance--based response function.
    """
    def __call__(self, iterator, input_activity, activity, strength, **params):
        cfs = iterator.flatcfs
        rows,cols = activity.shape
        euclidean_dist_mat = zeros((rows,cols),Float)
        for r in xrange(rows):
            for c in xrange(cols):
                flati = r*cols+c
                cf = cfs[flati]
                r1,r2,c1,c2 = cf.input_sheet_slice
                X = input_activity[r1:r2,c1:c2]
                diff = ravel(X) - ravel(cf.weights)
                euclidean_dist_mat.flat[flati] = L2norm(diff)

        max_dist = max(euclidean_dist_mat.ravel())
        activity *= 0.0
        activity += (max_dist - euclidean_dist_mat)
        activity *= strength



class CFPRF_ActivityBased(CFPResponseFn):
    """
    Calculate the activity of each unit nonlinearly based on the input activity.

    The activity is calculated from the input activity, the weights,
    and a strength that is a function of the input activity. This
    allows connections to have either an excitatory or inhibitory
    effect, depending on the activity entering the unit in question.
    
    The strength function is a generalized logistic curve (Richards'
    curve), a flexible function for specifying a nonlinear growth
    curve::
    
    y = l + ( u /(1 + b exp(-r (x - 2m)) ^ (1 / b)) )

    This function has five parameters::

    * l: the lower asymptote, i.e. the value at infinity;
    * u: the upper asymptote minus l, i.e. (u + l) is the value at minus infinity;
    * m: the time of maximum growth;
    * r: the growth rate;
    * b: affects near which asymptote maximum growth occurs.

    Richards, F.J. 1959 A flexible growth function for empirical use.
    J. Experimental Botany 10: 290--300, 1959.
    http://en.wikipedia.org/wiki/Generalised_logistic_curve
    """

    l = param.Number(default=-1.3,doc="Value at infinity")
    u = param.Number(default=1.2,doc="(u + l) is the value at minus infinity")
    m = param.Number(default=0.25,doc="Time of maximum growth.")
    r = param.Number(default=-200,doc="Growth rate, controls the gradient")
    b = param.Number(default=2,doc="Controls position of maximum growth")
    single_cf_fn = param.ClassSelector(ResponseFn,default=DotProduct(),doc="""
        ResponseFn to apply to each CF individually.""")
  
    def __call__(self, iterator, input_activity, activity, strength):
        single_cf_fn = self.single_cf_fn
        normalize_factor=max(input_activity.flat)
        
        for cf,r,c in iterator():
            r1,r2,c1,c2 = cf.input_sheet_slice
            X = input_activity[r1:r2,c1:c2]
            avg_activity=sum(X.flat)/len(X.flat)
            x=avg_activity/normalize_factor
            strength_fn=self.l+(self.u/(1+exp(-self.r*(x-2*self.m)))**(1.0/self.b))
            activity[r,c] = single_cf_fn(X,cf.weights)
            activity[r,c] *= strength_fn

__all__ = [
    "CFPRF_EuclideanDistance",
    "CFPRF_ActivityBased",
    "CFPRF_Plugin",
]



"""
TestFeatureMap

$Id$

"""


# To do:
# - Remove the commented line from when it was calling a function 
# - finish to transform the call to be to a procedure in the last test
# - Also clean up the original file to this purpose


import unittest
import copy

from topo.transferfn import PiecewiseLinear, DivisiveNormalizeL1
from topo.transferfn import DivisiveNormalizeL2, DivisiveNormalizeLinf
from topo.transferfn import DivisiveNormalizeLp, HomeostaticMaxEnt

from numpy.oldnumeric import array
from numpy.testing import assert_array_equal
from math import sqrt

from utils import assert_array_not_equal

class TestPiecewiseLinear(unittest.TestCase):

    def setUp(self):

        self.a1 = array([[0.5,-1.0,0.99],
                        [1.001,-0.001,0.6]])

        self.a2 = array([[1.0,-1.0,7.0],
                        [4.0,3.0,11.0]])

        self.fn1 = PiecewiseLinear()
        self.fn2 = PiecewiseLinear(lower_bound = 0.1,upper_bound = 0.5)    
        self.fn3 = PiecewiseLinear(lower_bound = 0, upper_bound = 10.0)
        
    def test_piecewiselinear(self):
        # Test as a procedure

        fn1_a1 = array([[0.5,0.0,0.99],
                       [1.0,0.0,0.6]])

        fn2_a1 = array([[1.0, 0.0, 1.0],
                       [1.0,0.0, 1.0]])

        fn3_a2 = array([[0.1,0.0,0.7],
                       [0.4,0.3,1.0]])


        self.fn1(self.a1)
        for item1,item2 in zip(self.a1.ravel(),fn1_a1.ravel()):
            self.assertAlmostEqual(item1, item2)

        self.a1 = array([[0.5,-1.0,0.99],
                        [1.001,-0.001,0.6]])
        self.fn2(self.a1)
        for item1,item2 in zip(self.a1.ravel(),fn2_a1.ravel()):
            self.assertAlmostEqual(item1, item2)

            
        self.fn3(self.a2)
        for item1,item2 in zip(self.a2.ravel(),fn3_a2.ravel()):
            self.assertAlmostEqual(item1, item2)
 
            
class TestDivisiveNormalizeL1(unittest.TestCase):
    
    def setUp(self):

        self.a1 = array([[0.3,0.6,0.7],
                        [0.8,0.4,0.2]])

        self.a2 = array([[1.0,-1.0,7.0],
                        [4.0,3.0,11.0]])

        self.fn1 = DivisiveNormalizeL1()
        self.fn2 = DivisiveNormalizeL1(norm_value=4.0)
               
    def test_divisive_sum_normalize(self):
        # Test as a procedure

        fn1_a1 = self.a1/3.0

        fn1_a2 = self.a2/27.0

        fn2_a1 = (self.a1/3.0)*4.0

        fn2_a2 = (self.a2/27.0)*4.0


        self.fn1(self.a1)
        for item1,item2 in zip(self.a1.ravel(),fn1_a1.ravel()):
            self.assertAlmostEqual(item1, item2)

        self.fn1(self.a2)
        for item1,item2 in zip(self.a2.ravel(),fn1_a2.ravel()):
            self.assertAlmostEqual(item1, item2)
            
        self.a1 = array([[0.3,0.6,0.7],
                        [0.8,0.4,0.2]])

        self.a2 = array([[1.0,-1.0,7.0],
                        [4.0,3.0,11.0]])
            
        self.fn2(self.a1)
        for item1,item2 in zip(self.a1.ravel(),fn2_a1.ravel()):
            self.assertAlmostEqual(item1, item2)

        self.fn2(self.a2)
        for item1,item2 in zip(self.a2.ravel(),fn2_a2.ravel()):
            self.assertAlmostEqual(item1, item2)


class TestDivisiveLengthNormalize(unittest.TestCase):
    
    def setUp(self):

        self.a1 = array([[0.3,0.6,0.7],
                        [0.8,0.4,0.2]])

        self.a2 = array([[1.0,-1.0,7.0],
                         [4.0,3.0,11.0],
                         [2.0,5.0,9.0]])

        self.fn1 = DivisiveNormalizeL2()        
        self.fn2 = DivisiveNormalizeL2(norm_value=4.0)
               
    def test_divisive_length_normalize(self):
        # Test as a procedure
        
        eucl_norm_a1 = sqrt(0.3**2+0.6**2+0.7**2+0.8**2+0.4**2+0.2**2)
        # eucl_norm_a2 = sqrt(307)
        
        fn1_a1 = self.a1/eucl_norm_a1
        fn1_a2 = self.a2/sqrt(307)
        fn2_a1 = (self.a1/eucl_norm_a1)*4.0
        fn2_a2 = (self.a2/sqrt(307))*4.0


        self.fn1(self.a1)
        for item1,item2 in zip(self.a1.ravel(),fn1_a1.ravel()):
            self.assertAlmostEqual(item1, item2)

        self.fn1(self.a2)
        for item1,item2 in zip(self.a2.ravel(),fn1_a2.ravel()):
            self.assertAlmostEqual(item1, item2)
            
        self.a1 = array([[0.3,0.6,0.7],
                        [0.8,0.4,0.2]])

        self.a2 = array([[1.0,-1.0,7.0],
                         [4.0,3.0,11.0],
                         [2.0,5.0,9.0]])

            
        self.fn2(self.a1)
        for item1,item2 in zip(self.a1.ravel(),fn2_a1.ravel()):
            self.assertAlmostEqual(item1, item2)

        self.fn2(self.a2)
        for item1,item2 in zip(self.a2.ravel(),fn2_a2.ravel()):
            self.assertAlmostEqual(item1, item2)

            
class TestDivisiveMaxNormalize(unittest.TestCase):
    
    def setUp(self):

        self.a1 = array([[0.3,0.6,0.7],
                        [0.8,0.4,0.2]])

        self.a2 = array([[1.0,-1.0,7.0],
                         [4.0,3.0,-11.0],
                         [2.0,5.0,9.0]])

        self.fn1 = DivisiveNormalizeLinf()        
        self.fn2 = DivisiveNormalizeLinf(norm_value=3.0)
               
    def test_divisive_max_normalize(self):
        # Test as a procedure:
        
        fn1_a1 = self.a1/0.8
        fn1_a2 = self.a2/11.0
        fn2_a1 = (self.a1/0.8)*3.0
        fn2_a2 = (self.a2/11.0)*3.0


        self.fn1(self.a1)
        for item1,item2 in zip(self.a1.ravel(),fn1_a1.ravel()):
            self.assertAlmostEqual(item1, item2)

        self.fn1(self.a2)
        for item1,item2 in zip(self.a2.ravel(),fn1_a2.ravel()):
            self.assertAlmostEqual(item1, item2)
                    
        self.a1 = array([[0.3,0.6,0.7],
                        [0.8,0.4,0.2]])

        self.a2 = array([[1.0,-1.0,7.0],
                         [4.0,3.0,-11.0],
                         [2.0,5.0,9.0]])
    
        self.fn2(self.a1)
        for item1,item2 in zip(self.a1.ravel(),fn2_a1.ravel()):
            self.assertAlmostEqual(item1, item2)

        self.fn2(self.a2)
        for item1,item2 in zip(self.a2.ravel(),fn2_a2.ravel()):
            self.assertAlmostEqual(item1, item2)


class TestDivisiveLpNormalize(unittest.TestCase):
    
    def setUp(self):

        self.a1 = array([[0.3,0.6,0.7],
                        [0.8,0.4,0.2]])

        self.a2 = array([[1.0,-1.0,7.0],
                         [4.0,3.0,-11.0],
                         [2.0,5.0,9.0]])

        # The default value of p is 2, so in this case, same as L2
        self.fn1 = DivisiveNormalizeLp()        
        self.fn2 = DivisiveNormalizeLp(p=3.0)
        self.fn3 = DivisiveNormalizeLp(p=4.0,norm_value=2.0)
        
    def test_divisive_lp_normalize(self):

        ### JCALERT! As already said above; this method does not work as a procedure
        ### because of a problem when using x *= factor instead of x = x * factor
        ### it is not understood why because it does work fine in all others transferfn?
        ### Therefore it is only tested as a function

        # Test as a function
        
        eucl_norm_a1 = sqrt(0.3**2+0.6**2+0.7**2+0.8**2+0.4**2+0.2**2)

        fn1_a1 = self.a1/eucl_norm_a1
        fn1_a2 = self.a2/sqrt(307)

        l3norm_a1 = pow(0.3**3+0.6**3+0.7**3+0.8**3+0.4**3+0.2**3,1.0/3.0)
        l3norm_a2 = pow(2.0+7.0**3+4.0**3+3.0**3+11.0**3+2.0**3+5.0**3+9.0**3,1.0/3.0)
        fn2_a1 = self.a1/l3norm_a1
        fn2_a2 = self.a2/l3norm_a2

        self.fn1(self.a1)
        self.fn1(self.a2)
        for item1,item2 in zip(self.a1.ravel(),fn1_a1.ravel()):
            self.assertAlmostEqual(item1, item2)
        for item1,item2 in zip(self.a2.ravel(),fn1_a2.ravel()):
            self.assertAlmostEqual(item1, item2)

        self.a1 = array([[0.3,0.6,0.7],
                        [0.8,0.4,0.2]])

        self.a2 = array([[1.0,-1.0,7.0],
                         [4.0,3.0,-11.0],
                         [2.0,5.0,9.0]])

        self.fn2(self.a1)
        self.fn2(self.a2)
        for item1,item2 in zip(self.a1.ravel(),fn2_a1.ravel()):
            self.assertAlmostEqual(item1, item2)
        for item1,item2 in zip(self.a2.ravel(),fn2_a2.ravel()):
            self.assertAlmostEqual(item1, item2)

        self.a1 = array([[0.3,0.6,0.7],
                        [0.8,0.4,0.2]])

        self.a2 = array([[1.0,-1.0,7.0],
                         [4.0,3.0,-11.0],
                         [2.0,5.0,9.0]])

        l4norm_a1 = pow(0.3**4.0+0.6**4.0+0.7**4.0+0.8**4.0+0.4**4.0+0.2**4.0,1.0/4.0)
        l4norm_a2 = pow(1.0**4.0+1.0**4.0+7.0**4.0+4.0**4.0+3.0**4.0+11.0**4.0+2.0**4.0+5.0**4.0+9.0**4.0,1.0/4.0)
        fn3_a1 = (self.a1/l4norm_a1)*2.0
        fn3_a2 = (self.a2/l4norm_a2)*2.0


        self.fn3(self.a1)
        self.fn3(self.a2)
        for item1,item2 in zip(self.a1.ravel(),fn3_a1.ravel()):
            self.assertAlmostEqual(item1, item2)
        for item1,item2 in zip(self.a2.ravel(),fn3_a2.ravel()):
            self.assertAlmostEqual(item1, item2)

        # The rest of this procedure might be redundant (already covered above)

        self.a1 = array([[0.3,0.6,0.7],
                        [0.8,0.4,0.2]])

        self.a2 = array([[1.0,-1.0,7.0],
                         [4.0,3.0,-11.0],
                         [2.0,5.0,9.0]])
        

        self.fn1(self.a1)
        for item1,item2 in zip(self.a1.ravel(),fn1_a1.ravel()):
            self.assertAlmostEqual(item1, item2)
        self.fn1(self.a2)    
        for item1,item2 in zip(self.a2.ravel(),fn1_a2.ravel()):
            self.assertAlmostEqual(item1, item2)
            
        self.a1 = array([[0.3,0.6,0.7],
                        [0.8,0.4,0.2]])

        self.a2 = array([[1.0,-1.0,7.0],
                         [4.0,3.0,-11.0],
                         [2.0,5.0,9.0]])
        
        self.fn2(self.a1)
        for item1,item2 in zip(self.a1.ravel(),fn2_a1.ravel()):
            self.assertAlmostEqual(item1, item2)
        self.fn2(self.a2)    
        for item1,item2 in zip(self.a2.ravel(),fn2_a2.ravel()):
            self.assertAlmostEqual(item1, item2)      

       
        
        self.a1 = array([[0.3,0.6,0.7],
                        [0.8,0.4,0.2]])

        self.a2 = array([[1.0,-1.0,7.0],
                         [4.0,3.0,-11.0],
                         [2.0,5.0,9.0]])

        self.fn3(self.a1)
        for item1,item2 in zip(self.a1.ravel(),fn3_a1.ravel()):
            self.assertAlmostEqual(item1, item2)
        self.fn3(self.a2)    
        for item1,item2 in zip(self.a2.ravel(),fn3_a2.ravel()):
            self.assertAlmostEqual(item1, item2)      



class TestTransferFnWithRandomState(unittest.TestCase):
    """Currently just tests whether random generator state is handled
    by state_push() and state_pop()."""

    def setUp(self):

        self.a = array([[0.3,0.6,0.7],
                        [0.8,0.4,0.2]])

        self.hme = HomeostaticMaxEnt()

    def test_random_state(self):

        a1 = copy.copy(self.a)

        ###
        # CEBALERT: hack because homeostaticmaxent doesn't get some
        # of its state until it's been called once!
        self.hme(a1)

        start_array = copy.copy(a1)

        ## save the state then generate some results...
        self.hme.state_push()

        self.hme(a1)
        res1 = copy.copy(a1)
        self.hme(a1)
        res2 = copy.copy(a1)

        # (check results do actually change so the test is valid)
        assert_array_not_equal(res1,res2)

        a2 = copy.copy(start_array) 

        ### ...then restore the state & check results are the same
        self.hme.state_pop()
        self.hme(a2)
        assert_array_equal(a2,res1)

        self.hme(a2)
        assert_array_equal(a2,res2)



cases = [TestPiecewiseLinear,
         TestDivisiveNormalizeL1,
         TestDivisiveLengthNormalize,
         TestDivisiveMaxNormalize,
         TestDivisiveLpNormalize,
         TestTransferFnWithRandomState]

suite = unittest.TestSuite()
suite.addTests(unittest.makeSuite(case) for case in cases)
              
if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=1).run(suite)
    


        



"""
A Sheet class for composing activity from different sheets into a
single activity matrix.  Primarily a simple example of how to make
a sheet class, but can also be useful.

$Id$
"""

from numpy.oldnumeric import zeros

from topo.base.sheet import Sheet
from topo.misc.util import Struct, NxN

class Composer(Sheet):
    """
    A Sheet that combines the activity of 2 or more other sheets into
    a single activity matrix.  When connecting a sheet to a composer,
    you can specify the location at which that sheet's input will be
    mapped into the composer by adding the 'origin' argument to the
    connect() call e.g.:

    sim.connect(input_sheet.name,composer.name,delay=1, origin=(0.25,0.25))

    will cause (0,0) on input sheet's activity to map to (0.25,0.25)
    on composer's activity.

    """

    dest_ports=None # Allows connections to come in on any port

    def __init__(self,**params):
        super(Composer,self).__init__(**params)        
        self.inputs = {}
        self.__dirty = False

    def port_configure(self,port,**config):
        """
        Configure a specific input port.
        
        origin = (default (0,0)) The offset in the output matrix where
        this port's input should be placed.
        """
        if not port in self.ports:
            self.ports[port] = {}

        for k,v in config.items():
            self.ports[port][k] = v

    def _dest_connect(self,proj,origin=(0,0)):
        super(Composer,self)._dest_connect(proj)
        self.inputs[(proj.src.name,proj.src_port)] = Struct(origin=origin)
    
    def process_current_time(self):
        if self.__dirty:        
            self.send_output(data=self.activity) 
            self.activity = zeros(self.activity.shape)+0.0
            self.__dirty=False
           
    def input_event(self,conn,data):

        self.verbose("Received %s input from %s." % (NxN(data.shape),conn.src))

        self.__dirty = True

        in_rows, in_cols = data.shape

        # compute the correct position of the input in the buffer
        start_row,start_col = self.sheet2matrixidx(*self.inputs[(conn.src.name,conn.src_port)].origin)
        row_adj,col_adj = conn.src.sheet2matrixidx(0,0)

        self.debug("origin (row,col) = "+`(start_row,start_col)`)
        self.debug("adjust (row,col) = "+`(row_adj,col_adj)`)

        start_row -= row_adj
        start_col -= col_adj

        # the maximum bounds
        max_row,max_col = self.activity.shape

        self.debug("max_row = %d, max_col = %d" % (max_row,max_col))
        self.debug("in_rows = %d, in_cols = %d" % (in_rows,in_cols))

        end_row = start_row+in_rows
        end_col = start_col+in_cols

        # if the input goes outside the activity, clip it
        left_clip = -min(start_col,0)
        top_clip  = -min(start_row,0)
        right_clip = max(end_col,max_col) - max_col
        bottom_clip = max(end_row,max_row) - max_row

        start_col += left_clip
        start_row += top_clip
        end_col -= right_clip
        end_row -= bottom_clip

        self.debug("start_row = %d,start_col = %d" % (start_row,start_col))
        self.debug("end_row = %d,end_col = %d" % (end_row,end_col))
        self.debug("left_clip = %d" % left_clip)
        self.debug("right_clip = %d" % right_clip)
        self.debug("top_clip = %d" % top_clip)
        self.debug("bottom_clip = %d" % bottom_clip)
        self.debug("activity shape = %s" % NxN(self.activity.shape))

        self.activity[start_row:end_row, start_col:end_col] += data[top_clip:in_rows-bottom_clip,
                                                                      left_clip:in_cols-right_clip]


__all__ = [
    "Composer",
]



"""
GeneratorSheet: a sheet with a pattern generator.


$Id$
"""
__version__='$Revision: 7629 $'


import param
from topo.base.sheet import Sheet
from topo.base.patterngenerator import PatternGenerator,Constant
from topo.base.simulation import FunctionEvent, PeriodicEventSequence


# JLALERT: This sheet should have override_plasticity_state/restore_plasticity_state
# functions that call override_plasticity_state/restore_plasticty_state on the
# sheet output_fn and input_generator output_fn.
class GeneratorSheet(Sheet):
    """
    Sheet for generating a series of 2D patterns.

    Typically generates the patterns by choosing parameters from a
    random distribution, but can use any mechanism.
    """

    src_ports=['Activity']
    
    period = param.Number(default=1,bounds=(0,None), inclusive_bounds=(False, True), constant=True, doc=
        "Delay (in Simulation time) between generating new input patterns.")
    
    phase  = param.Number(default=0.05,doc=
        """
        Delay after the start of the Simulation (at time zero) before
        generating an input pattern.  For a clocked, feedforward simulation, 
        one would typically want to use a small nonzero phase and use delays less
        than the user-visible step size (typically 1.0), so that inputs are
        generated and processed before this step is complete.
        """)
    
    input_generator = param.ClassSelector(PatternGenerator,default=Constant(),
        doc="""Specifies a particular PatternGenerator type to use when creating patterns.""")

    
    def __init__(self,**params):
        super(GeneratorSheet,self).__init__(**params)
        self.input_generator_stack = []
        self.set_input_generator(self.input_generator)


    def set_input_generator(self,new_ig,push_existing=False):
        """
        Set the input_generator, overwriting the existing one by default.

        If push_existing is false, the existing input_generator is
        discarded permanently.  Otherwise, the existing one is put
        onto a stack, and can later be restored by calling
        pop_input_generator.
        """

        if push_existing:
            self.push_input_generator()

        # CEBALERT: replaces any bounds specified for the
        # PatternGenerator with this sheet's own bounds. When
        # PatternGenerators can draw patterns into supplied
        # boundingboxes, should remove this.
        new_ig.set_matrix_dimensions(self.bounds, self.xdensity, self.ydensity)
        self.input_generator = new_ig


    def push_input_generator(self):
        """Push the current input_generator onto a stack for future retrieval."""
        self.input_generator_stack.append(self.input_generator)

        # CEBALERT: would be better to reorganize code so that
        # push_input_generator must be supplied with a new generator.
        # CEBALERT: presumably we can remove this import.
        from topo.base.patterngenerator import Constant
        self.set_input_generator(Constant()) 

               
    def pop_input_generator(self):
        """
        Discard the current input_generator, and retrieve the previous one from the stack.

        Warns if no input_generator is available on the stack.
        """
        if len(self.input_generator_stack) >= 1:
            self.set_input_generator(self.input_generator_stack.pop())
        else:
            self.warning('There is no previous input generator to restore.')

    def generate(self):
        """
        Generate the output and send it out the Activity port.
        """
        self.verbose("Generating a new pattern")

        # JABALERT: What does the [:] achieve here?  Copying the
        # values, instead of the pointer to the array?  Is that
        # guaranteed?
        self.activity[:] = self.input_generator()

        if self.apply_output_fns:
            for of in self.output_fns:
                of(self.activity)
        self.send_output(src_port='Activity',data=self.activity)
                                                        
              
    def start(self):
        assert self.simulation

        if self.period > 0:
            # if it has a positive period, then schedule a repeating event to trigger it
            e=FunctionEvent(0,self.generate)
            now = self.simulation.time()
            self.simulation.enqueue_event(PeriodicEventSequence(now+self.simulation.convert_to_time_type(self.phase),self.simulation.convert_to_time_type(self.period),[e]))

    def input_event(self,conn,data):
        raise NotImplementedError



"""
Unit test for Parameterized.


$Id$
"""

import unittest

import param

# CEBALERT: not anything like a complete test of Parameterized!

import random
class _SomeRandomNumbers(object):
    def __call__(self):
        return random.random()


class TestPO(param.Parameterized):
    inst = param.Parameter(default=[1,2,3],instantiate=True)
    notinst = param.Parameter(default=[1,2,3],instantiate=False)
    const = param.Parameter(default=1,constant=True)
    ro = param.Parameter(default="Hello",readonly=True)
    ro2 = param.Parameter(default=object(),readonly=True,instantiate=True)

    dyn = param.Dynamic(default=1)

class AnotherTestPO(param.Parameterized):
    instPO = param.Parameter(default=TestPO(),instantiate=True)
    notinstPO = param.Parameter(default=TestPO(),instantiate=False)


class TestAbstractPO(param.Parameterized):
    __abstract = True


class TestParamInstantiation(AnotherTestPO):
    instPO = param.Parameter(default=AnotherTestPO(),instantiate=False)
    

class TestParameterized(unittest.TestCase):

    def test_constant_parameter(self):
        """Test that you can't set a constant parameter after construction."""
        testpo = TestPO(const=17)
        self.assertEqual(testpo.const,17)
        self.assertRaises(TypeError,setattr,testpo,'const',10)

        # check you can set on class
        TestPO.const=9
        testpo = TestPO()
        self.assertEqual(testpo.const,9)

    def test_readonly_parameter(self):
        """Test that you can't set a read-only parameter on construction or as an attribute."""
        testpo = TestPO()
        self.assertEqual(testpo.ro,"Hello")

        # CB: couldn't figure out how to use assertRaises
        try:
            t = TestPO(ro=20)
        except TypeError:
            pass
        else:
            raise AssertionError("Read-only parameter was set!")

        t=TestPO()
        self.assertRaises(TypeError,setattr,t,'ro',10)

        # check you cannot set on class
        self.assertRaises(TypeError,setattr,TestPO,'ro',5)

        self.assertEqual(testpo.params()['ro'].constant,True)

        # check that instantiate was ignored for readonly
        self.assertEqual(testpo.params()['ro2'].instantiate,False)
        


    def test_basic_instantiation(self):
        """Check that instantiated parameters are copied into objects."""

        testpo = TestPO()

        self.assertEqual(testpo.inst,TestPO.inst)
        self.assertEqual(testpo.notinst,TestPO.notinst)

        TestPO.inst[1]=7
        TestPO.notinst[1]=7
        
        self.assertEqual(testpo.notinst,[1,7,3])
        self.assertEqual(testpo.inst,[1,2,3])


    def test_more_instantiation(self):
        """Show that objects in instantiated Parameters can still share data."""
        anothertestpo = AnotherTestPO()

        ### CB: AnotherTestPO.instPO is instantiated, but
        ### TestPO.notinst is not instantiated - so notinst is still
        ### shared, even by instantiated parameters of AnotherTestPO.
        ### Seems like this behavior of Parameterized could be
        ### confusing, so maybe mention it in documentation somewhere.
        TestPO.notinst[1]=7
        # (if you thought your instPO was completely an independent object, you
        # might be expecting [1,2,3] here)
        self.assertEqual(anothertestpo.instPO.notinst,[1,7,3])


    def test_instantiation_inheritance(self):
        """Check that instantiate=True is always inherited (SF.net #2483932)."""
        t = TestParamInstantiation()
        assert t.params('instPO').instantiate is True
        assert isinstance(t.instPO,AnotherTestPO)
        

    def test_abstract_class(self):
        """Check that a class declared abstract actually shows up as abstract."""
        self.assertEqual(TestAbstractPO.abstract,True)
        self.assertEqual(TestPO.abstract,False)


    def test_params(self):
        """Basic tests of params() method."""


        # CB: test not so good because it requires changes if params
        # of PO are changed
        assert 'name' in param.Parameterized.params()
        assert 'print_level' in param.Parameterized.params()
        assert len(param.Parameterized.params())==2

        ## check for bug where subclass Parameters were not showing up
        ## if params() already called on a super class.
        assert 'inst' in TestPO.params()
        assert 'notinst' in TestPO.params()

        ## check caching
        assert param.Parameterized.params() is param.Parameterized().params(), "Results of params() should be cached." # just for performance reasons


    def test_state_saving(self):
        t = TestPO(dyn=_SomeRandomNumbers())
        g = t.get_value_generator('dyn')
        g._Dynamic_time_fn=None
        assert t.dyn!=t.dyn
        orig = t.dyn
        t.state_push()
        t.dyn
        assert t.inspect_value('dyn')!=orig
        t.state_pop()
        assert t.inspect_value('dyn')==orig



from param import parameterized

class some_fn(param.ParameterizedFunction):
   num_phase = param.Number(18)
   frequencies = param.List([99])
   scale = param.Number(0.3)

   def __call__(self,**params_to_override):
       params = parameterized.ParamOverrides(self,params_to_override)
       num_phase = params['num_phase']
       frequencies = params['frequencies']
       scale = params['scale']
       return scale,num_phase,frequencies


instance = some_fn.instance() 

class TestParameterizedFunction(unittest.TestCase):

    def _basic_tests(self,fn):
        self.assertEqual(fn(),(0.3,18,[99]))
        self.assertEqual(fn(frequencies=[1,2,3]),(0.3,18,[1,2,3]))
        self.assertEqual(fn(),(0.3,18,[99]))
        
        fn.frequencies=[10,20,30]
        self.assertEqual(fn(frequencies=[1,2,3]),(0.3,18,[1,2,3]))
        self.assertEqual(fn(),(0.3,18,[10,20,30]))

    def test_parameterized_function(self):
        self._basic_tests(some_fn)

    def test_parameterized_function_instance(self):
        self._basic_tests(instance)

    def test_pickle_instance(self):
        import pickle
        s = pickle.dumps(instance)
        instance.scale=0.8
        i = pickle.loads(s)
        self.assertEqual(i(),(0.3,18,[10,20,30]))

        
suite = unittest.TestSuite()
cases = [TestParameterized,TestParameterizedFunction]
suite.addTests([unittest.makeSuite(case) for case in cases])
